@@ -1,9 +1,11 @@
 /*
+ *  This file is part of X-File Manager XFM
+ *  ----------------------------------------------------------------------
  *  Enhancements to the X-File Manager XFM-1.3.2 (FileList Widget)
- *  --------------------------------------------------------------
  
  *  Copyright (C) 1997  by Till Straumann   <strauman@sun6hft.ee.tu-berlin.de>
-
+ *  modified 2005,2006 by Bernhard R. Link
+ *  ----------------------------------------------------------------------
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -16,18 +18,22 @@
 
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
+#include <xfmconfig.h>
+
 #include <sys/stat.h>
+#include <assert.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "global.h"
+
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
 #include "FileListP.h"
-#include <X11/Xaw/XawInit.h>
+#include <X11/Xaw3d/XawInit.h>
 
 #define FL (flw->fileList)
 
@@ -65,7 +71,6 @@ static Boolean flSetValues(Widget,Widget,Widget,ArgList,Cardinal*);
 /* private procedures */
 static void ReleaseGCs(FileListWidget);
 static void GetGCs(FileListWidget);
-static void GetDefaultFileType(FileRec*);
 
 /* Event Handlers */
 static void FakeEntryLeave(Widget,XtPointer,XEvent*,Boolean *ctd);
@@ -77,16 +82,16 @@ static void DiscardMotion(Widget,XtPointer,XEvent*,Boolean *ctd);
 static void EnableMotion(XtPointer,XtIntervalId *);
 
 /* Actions */
-static void DispatchDirExeFile(Widget, XEvent*, String*, Cardinal* );
-static void DispatchDirExeFilePopup(Widget, XEvent*, String*, Cardinal* );
+static void DispatchDirFile(Widget, XEvent*, String*, Cardinal* );
+static void GrabMouseForPopup(Widget, XEvent *, String *, Cardinal *);
 static void Notify(Widget, XEvent*, String*, Cardinal*);
 
 
 static XtActionsRec actions[] =
 {
   /* {name, procedure}, */
-    {"dispatchDirExeFile",	DispatchDirExeFile},
-    {"dispatchDirExeFilePopup",	DispatchDirExeFilePopup},
+    {"dispatchDirFile",		DispatchDirFile},
+    {"grabMouseForPopup",	GrabMouseForPopup},
     {"notify",			Notify},
 
 };
@@ -156,15 +161,12 @@ static void ClassInitialize(void)
 {
  XawInitializeWidgetSet();
 
- XtRegisterGrabAction(DispatchDirExeFilePopup, 
+ XtRegisterGrabAction(GrabMouseForPopup, 
 			True, ButtonPressMask | ButtonReleaseMask,
 			GrabModeAsync, GrabModeAsync);
 }
 
-static void flInitialize(treq,tnew,args,n_args)
- Widget	  treq,tnew;
- ArgList  args;
- Cardinal *n_args;
+static void flInitialize(UNUSED(Widget treq),Widget tnew,UNUSED(ArgList args),UNUSED(Cardinal *n_args))
 {
  FileRec *file;
  int  i,tmp,width;
@@ -184,12 +186,7 @@ static void flInitialize(treq,tnew,args,n_args)
 	file->icon.form=(Widget)0;
 	file->icon.arrow=(Widget)0;
 	file->icon.label=(Widget)0;
-#ifdef MAGIC_HEADERS
-	if ((file->type=fileType(file->name,file->magic_type))==0) 
-	  GetDefaultFileType(file);
-#else
-	GetDefaultFileType(file);
-#endif
+	assert(file->mime_type != NULL );
  }
  width+=XTextWidth(FL.font,"[]",2);
  width+=2*FL.border_width;
@@ -230,8 +227,8 @@ static void flInitialize(treq,tnew,args,n_args)
 
 #define NF (new->fileList)
 #define CF (current->fileList)
-static Boolean flSetValues(Widget c, Widget r, Widget super,
-	ArgList args, Cardinal *nargs)
+static Boolean flSetValues(Widget c, UNUSED(Widget r), Widget super,
+	UNUSED(ArgList args), UNUSED(Cardinal *nargs))
 {
 Boolean rval=False,warn=False;
 FileListWidget current=(FileListWidget)c;
@@ -270,12 +267,11 @@ return rval;
 
 /* Actions */
 
-/*ARGSUSED*/
-static void Notify( Widget w, XEvent *ev, String *args, Cardinal *nargs)
+static void Notify( Widget w, UNUSED(XEvent *ev), String *args, Cardinal *nargs)
 {
 FileListWidget flw=(FileListWidget)w;
 String *alist=(String*) XtMalloc(*nargs+1);
-int	i;
+Cardinal i;
 
 alist[*nargs]=0;
 for (i=0; i<*nargs; i++) alist[i]=args[i];
@@ -285,34 +281,36 @@ XtCallCallbackList(w,FL.notify_cbl,(XtPointer)alist);
 XtFree((char*)alist);
 }
 
-/*ARGSUSED*/
-static void DispatchDirExeFilePopup( Widget w, XEvent *ev, String *args, Cardinal *nargs)
+static void GrabMouseForPopup( Widget w, XEvent *ev, String *args, Cardinal *nargs)
 {
- DispatchDirExeFile(w,ev,args,nargs);
+	if (nargs == NULL || *nargs<1) {
+		XtAppWarning(XtWidgetToApplicationContext(w),
+				"DispatchDirFile() needs min. 1 argument");
+		return;
+	}
+	XtCallActionProc(w,args[0],ev,args+1,*nargs-1);
 }
 
-/*ARGSUSED*/
-static void DispatchDirExeFile( Widget w, XEvent *ev, String *args, Cardinal *nargs)
+static void DispatchDirFile( Widget w, XEvent *ev, String *args, Cardinal *nargs)
 {
 FileListWidget flw=(FileListWidget)w;
-int i,idx;
+int i; unsigned int idx;
 FileRec *file;
 
-if (*nargs<4) {
+if (nargs == NULL || *nargs<3) {
 	XtAppWarning(XtWidgetToApplicationContext(w),
-		"DispatchDirExeFile() needs min. 4 arguments");
+		"DispatchDirFile() needs min. 3 arguments");
 	return;
 }
 i=FileListFindEntry(flw,ev);
-if (i==-1) idx=3;
+if (i==-1) idx=2;
 else {
   file=FL.files[i];
   if (S_ISDIR(file->stats.st_mode)) idx=0;
-  else if (file->stats.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) idx=1;
-  else idx=2;
+  else idx=1;
 }
 if (*args[idx]!=(char)0)
-  XtCallActionProc(w,args[idx],ev,args+4,idx-4);
+  XtCallActionProc(w,args[idx],ev,args+3,*nargs-3);
 }
 
 
@@ -381,51 +379,36 @@ void FileListRefreshItem(FileListWidget flw, int item)
 	True);
 }
  
-static void CatchEntryLeave(Widget w, XtPointer cld, XEvent* ev, Boolean *ctd)
+static void CatchEntryLeave(Widget w, UNUSED(XtPointer cld), XEvent* ev, Boolean *ctd)
 {
-int x,y,entry;
 FileListWidget flw=(FileListWidget)w;
 FileListWidgetClass classptr=(FileListWidgetClass)XtClass((Widget)flw);
 
 if (ev->xcrossing.send_event) return;
 
-entry=-1;
-*ctd=False; /* eat up the original event */
+*ctd=True; /* no longer eat up event */
 if (ev->type==EnterNotify) {
-    x=ev->xcrossing.x;
-    y=ev->xcrossing.y;
-    entry=classptr->fileList_class.findEntry(flw,x,y,ev);
+    int x=ev->xcrossing.x;
+    int y=ev->xcrossing.y;
+    int entry=classptr->fileList_class.findEntry(flw,x,y,ev);
     if (entry!=-1) {
       FL.enter_x=x; FL.enter_y=y;
-	  /* `kwm' inserts Entry/Leave events before a buttonPress;
-	   * which disturbs the whole translation stuff. We just
-	   * throw away enter/leave events that don't origin from
-	   * our SendEvents (The semantics of Entry/Leave for FileLists
-	   * is changed anyway: these events refer to entry/leaves to
-	   * the items and not for the whole window anymore).
-	   */
-      /* *ctd=True; */
-      /* don't do SendEvent, because it would go back to the end of the queue;
-       * in the queue might be other events that must be handled _after_ the
-       * faked leave of the file item.
-       */
     }
+    FL.pointer_entry=entry;
   } else {
     if (FL.pointer_entry!=-1) {
       ev->xcrossing.x=FL.enter_x;
       ev->xcrossing.y=FL.enter_y;
-	  /* for kwm behavior see above */
-      /* *ctd=True; */ /* see above */
     }
+    FL.pointer_entry=-1;
   }
-  FL.pointer_entry=entry;
 }
 
 /* A button was pressed, suppress propagation of motion events for 
  * some time; that way drags can be separated from clicks if the
  * user just moved the pointer a bit while clicking
  */
-static void StartTimer(Widget w, XtPointer cld, XEvent* ev, Boolean *ctd)
+static void StartTimer(Widget w, UNUSED(XtPointer cld), UNUSED(XEvent* ev), UNUSED(Boolean *ctd))
 {
  FileListWidget flw=(FileListWidget)w;
  if (FL.timer_running) XtRemoveTimeOut(FL.timeout_id);
@@ -439,13 +422,13 @@ static void StartTimer(Widget w, XtPointer cld, XEvent* ev, Boolean *ctd)
 /* If a timer is still running, don't continue to dispatch
  * ButtonMotion events.
  */
-static void DiscardMotion(Widget w, XtPointer cld, XEvent* ev, Boolean *ctd)
+static void DiscardMotion(Widget w, UNUSED(XtPointer cld), UNUSED(XEvent* ev), Boolean *ctd)
 {
 FileListWidget flw=(FileListWidget)w;
 if (FL.timer_running) *ctd=False;
 }
 
-static void FakeEntryLeave(Widget w, XtPointer cld, XEvent* ev, Boolean *ctd)
+static void FakeEntryLeave(Widget w, UNUSED(XtPointer cld), XEvent* ev, UNUSED(Boolean *ctd))
 {
 int x,y,entry;
 FileListWidget flw=(FileListWidget)w;
@@ -511,7 +494,7 @@ if ((FL.pointer_entry=entry)!=-1) {
  * we guess that the user really intends to drag something
  * if he/she moves the pointer now.
  */ 
-static void EnableMotion(XtPointer cld, XtIntervalId *id)
+static void EnableMotion(XtPointer cld, UNUSED(XtIntervalId *id))
 {
 FileListWidget flw=(FileListWidget)cld;
 FL.timer_running=False; /*Timer just expired*/
@@ -547,43 +530,3 @@ static void GetGCs(FileListWidget flw)
  vals.background=FL.highlight_pixel;
  FL.gc_invert=XtGetGC((Widget)flw,mask,&vals);
 }
-
-static void GetDefaultFileType(FileRec *file)
-{
-   if (S_ISLNK(file->stats.st_mode)) {
-	file->type=&builtin_types[BLACKHOLE_T];
-	return;
-   } else if (S_ISDIR(file->stats.st_mode)) {
-	if (file->sym_link) {
-	    file->type=&builtin_types[DIRLNK_T];
-	    return;
-	} else if (!strcmp(file->name, "..")) {
-	    file->type=&builtin_types[UPDIR_T];
-	    return;
-	} else {
-	    file->type=&builtin_types[DIR_T];
-	    return;
-	}
-   } else if (file->stats.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) {
-	if (file->sym_link) {
-	  file->type=&builtin_types[EXECLNK_T];
-	  return;
-	} else {
-	  file->type=&builtin_types[EXEC_T];
-	  return;
-	}
-    } else {
-#ifndef MAGIC_HEADERS
-	file->type=fileType(file->name);
-	if (file->type) return;
-#endif
-	if (file->sym_link) {
-	  file->type=&builtin_types[SYMLNK_T];
-	  return;
-	} else {
-	  file->type=&builtin_types[FILE_T];
-	  return;
-	}
-    }
-/* should never get here */
-}
