@@ -1,80 +1,74 @@
-/*---------------------------------------------------------------------------
-  Module FmAwActions
+/*
+ *  This file is part of X-File Manager XFM
+ *  ----------------------------------------------------------------------
+  FmAwActions.c
   
   (c) Simon Marlow 1990-92
   (c) Albert Graef 1994
 
-  Action procedures for widgets in the application window
----------------------------------------------------------------------------*/
+  modified 2004,2005,2006 by Bernhard R. Link (see Changelog)
 
+  Action procedures for widgets in the application window
+ *  ----------------------------------------------------------------------
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <xfmconfig.h>
+
+#include <assert.h>
 #include <string.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
-#include <X11/Xaw/Toggle.h>
+#include <X11/Xaw3d/Toggle.h>
+
+#include "global.h"
 
 #include "Am.h"
+#include "execute.h"
 
 /*---------------------------------------------------------------------------
   PRIVATE FUNCTIONS
 ---------------------------------------------------------------------------*/
 
-/* determine the cursor type for an application */
+static void awDragGrab(Boolean dropable) {
+	Cursor c;
 
-static Cursor cursorType(char *name, char *directory, char *fname)
-{
-  if (!*fname)
-    return curs[EXEC_CUR];
-  else {
-    struct stat stats;
-    int l = *directory?strlen(directory):strlen(user.home);
-    char *path = (char *)alloca(2+strlen(fname)+l);
-    
-    strcpy(path, *directory?directory:user.home);
+	if (dropable)
+		c = curs[EXEC_CUR];
+	else
+		c = curs[NOENTRY_CUR];
 
-    if (l) {
-      if (path[l-1] != '/')
-	path[l++] = '/';
-      strcpy(path+l, fname);
-    } else
-      strcpy(path, fname);
-
-    if (lstat(path, &stats))
-      return curs[FILE_CUR];
-    else if (S_ISDIR(stats.st_mode))
-      return curs[DIR_CUR];
-    else if (stats.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
-      return curs[EXEC_CUR];
-    else
-      return curs[FILE_CUR];
-  }
+	XGrabPointer(XtDisplay(aw.shell), XtWindow(aw.shell), True,
+			EnterWindowMask | LeaveWindowMask | ButtonReleaseMask, 
+			GrabModeAsync, GrabModeAsync, None, c, CurrentTime);
 }
 
-/*---------------------------------------------------------------------------*/
-
 /* perform a click-drag, returning the widget that the user released the 
    button on */
 
 static Boolean dragging_app = False;
 
-static Widget drag(Widget w, int button,
+static Widget drag(Widget w, unsigned int button,
 		   Boolean *on_root_return)
 {
-  Cursor c;
   XEvent e;
-  int i, x, y;
+  int x, y;
   Window child;
 
-  if (aw.n_selections == 1) {
-    for (i=0; !aw.apps[i].selected && i<aw.n_apps;
-	 i++);
-    c = cursorType(aw.apps[i].name, aw.apps[i].directory, aw.apps[i].fname);
-  } else
-    c = curs[FILES_CUR];
-
-  XGrabPointer(XtDisplay(w), XtWindow(w), True,
-	       EnterWindowMask | LeaveWindowMask | ButtonReleaseMask, 
-	       GrabModeAsync, GrabModeAsync, None, c, CurrentTime);
+  awDragGrab(True);
 
   freeze = dragging_app = True;
   
@@ -115,7 +109,7 @@ static Widget drag(Widget w, int button,
 
 int findAppWidget(Widget w)
 {
-  int i;
+  Cardinal i;
 
   for (i=0; i<aw.n_apps; i++)
     if (aw.apps[i].toggle == w)
@@ -156,15 +150,27 @@ void appPopup(Widget w, XEvent *event, String *params, Cardinal *num_params)
       grayOut(app_popup_items[7]);
       grayOut(app_popup_items[10]);
     } else {
-      fillIn(app_popup_items[3]);
+      if (aw.readonly) {
+        grayOut(app_popup_items[3]);
+        grayOut(app_popup_items[7]);
+      } else {
+        fillIn(app_popup_items[3]);
+	fillIn(app_popup_items[7]);
+      }
       fillIn(app_popup_items[4]);
-      fillIn(app_popup_items[7]);
       fillIn(app_popup_items[10]);
     }
-    if (stat(resources.app_clip, &stats))
+    if (stat(resources.app_clip, &stats) || aw.readonly)
       grayOut(app_popup_items[5]);
     else
       fillIn(app_popup_items[5]);
+    if (aw.readonly) {
+      grayOut(app_popup_items[0]);
+      grayOut(app_popup_items[1]);
+    } else {
+      fillIn(app_popup_items[0]);
+      fillIn(app_popup_items[1]);
+    }
 
     XQueryPointer(dpy, DefaultRootWindow(dpy), &root, &child, &x, &y, 
 		  &x_win, &y_win, &mask);
@@ -174,6 +180,15 @@ void appPopup(Widget w, XEvent *event, String *params, Cardinal *num_params)
   
     XtPopupSpringLoaded(app_popup_widget);
   } else {
+    if (aw.readonly) {
+	    grayOut(app_popup_items1[0]);
+	    grayOut(app_popup_items1[2]);
+	    grayOut(app_popup_items1[5]);
+    } else {
+	    fillIn(app_popup_items1[0]);
+	    fillIn(app_popup_items1[2]);
+	    fillIn(app_popup_items1[5]);
+    }
     XQueryPointer(dpy, DefaultRootWindow(dpy), &root, &child, &x, &y, 
 		  &x_win, &y_win, &mask);
   
@@ -186,73 +201,63 @@ void appPopup(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
 /*---------------------------------------------------------------------------*/
 
-void appMaybeHighlight(Widget w, XEvent *event, String *params, 
-		       Cardinal *num_params)
+void appMaybeHighlight(Widget w, XEvent *event,
+		       UNUSED(String *params), UNUSED(Cardinal *num_params))
 {
   int i;
 
   if (dragging) {
     i = findAppWidget(w);
-    if (*aw.apps[i].drop_action)
+    if (*aw.apps[i].drop_action) {
       XtCallActionProc(w, "highlight", event, NULL, 0);
-  } else if (dragging_app)
+      drag_set_dropable(XtDisplay(w), True);
+    } else {
+      drag_set_dropable(XtDisplay(w), False);
+    }
+  } else if (dragging_app) {
     XtCallActionProc(w, "highlight", event, NULL, 0);
+    awDragGrab(True);
+  }
 }
 
 /*---------------------------------------------------------------------------*/
 
-void runApp(Widget w, XEvent *event, String *params, Cardinal *num_params)
+void runApp(Widget w, UNUSED(XEvent *event),
+	    UNUSED(String *params), UNUSED(Cardinal *num_params))
 {
   int i;
-  char **argv;
-  char directory[MAXPATHLEN];
+  char *directory;
 
   i = findAppWidget(w);
-  strcpy(directory, aw.apps[i].directory);
-  if (*directory)
-    fnexpand(directory);
-  else
-    strcpy(directory, user.home);
+  directory = expanddirectory(aw.apps[i].directory);
 
-  if (*aw.apps[i].push_action)
-    if (!strcmp(aw.apps[i].push_action, "EDIT"))
-      doEdit(directory, aw.apps[i].fname);
-    else if (!strcmp(aw.apps[i].push_action, "VIEW"))
-      doView(directory, aw.apps[i].fname);
+  if (*aw.apps[i].push_action) {
+    if (!strncmp(aw.apps[i].push_action, "EDIT ", 5))
+      doEdit(directory, aw.apps[i].push_action+5);
+    else if (!strncmp(aw.apps[i].push_action, "VIEW ", 5))
+      doView(directory, aw.apps[i].push_action+5);
     else if (!strcmp(aw.apps[i].push_action, "OPEN")) {
-      int l = strlen(directory);
-      if (directory[l-1] != '/')
-	directory[l++] = '/';
-      strcpy(directory+l, aw.apps[i].fname);
       newFileWindow(directory, resources.default_display_type,
 		    False);
     } else if (!strcmp(aw.apps[i].push_action, "LOAD")) {
-      int j, l = strlen(directory);
       zzz();
-      pushApplicationsFile();
-      strcpy(resources.app_file, directory);
-      if (resources.app_file[l-1] != '/')
-	resources.app_file[l++] = '/';
-      strcpy(resources.app_file+l, aw.apps[i].fname);
-      for(j=0; j<aw.n_apps; j++)
-	freeApplicationResources(&aw.apps[j]);
-      XTFREE(aw.apps);
-      readApplicationData(resources.app_file);
-      updateApplicationDisplay();
+      AmPushLoadFile(directory,False);
       wakeUp();
     } else {
       char *action = varPopup(aw.apps[i].icon_bm, aw.apps[i].push_action);
-      if (!action) return;
-      if (*aw.apps[i].fname)
-	argv = makeArgv2(action, aw.apps[i].fname);
-      else
-	argv = makeArgv(action);
-      executeApplication(user.shell, directory, argv);
-      freeArgv(argv);
+      const char **args = makeShellArgs(action,NULL);
+      execute_external_program(args[0],directory,args,NULL,NULL);
+      XTFREE(args);
+      XtFree(action);
     }
+  }
 }
 
 /*---------------------------------------------------------------------------*/
+static void appEndMoveInBox(void);
+static void appEndCopyInBox(void);
+static void appEndCopy(int i);
+static void appEndMove(Cardinal i);
 
 void appBeginDrag(Widget w, XEvent *event, String *params, 
 		  Cardinal *num_params)
@@ -287,19 +292,20 @@ void appBeginDrag(Widget w, XEvent *event, String *params,
       appEndMoveInBox();
     else
       appEndCopyInBox();
-  else if ((i = findAppWidget(w)) != -1)
+  else if ((i = findAppWidget(w)) >= 0) {
     if (move)
       appEndMove(i);
     else
       appEndCopy(i);
+  }
 }
   
 /*---------------------------------------------------------------------------*/
 
-void appEndMoveInBox(void)
+static void appEndMoveInBox(void)
 {
   char s[0xff];
-  int j, k, l;
+  Cardinal j, k, l;
   AppRec *apps;
 
   if (aw.n_selections == 0) return;
@@ -312,6 +318,7 @@ void appEndMoveInBox(void)
   }
 
   apps = (AppRec*) XtMalloc(aw.n_apps*sizeof(AppRec));
+  assert(aw.n_apps >= aw.n_selections);
   for (j = k = 0, l = aw.n_apps - aw.n_selections; j < aw.n_apps; j++)
     if (aw.apps[j].selected)
       memcpy(&apps[l++], &aw.apps[j], sizeof(AppRec));
@@ -320,15 +327,15 @@ void appEndMoveInBox(void)
   XTFREE(aw.apps);
   aw.apps = apps;
   updateApplicationDisplay();
-  writeApplicationData(resources.app_file);
+  writeApplicationData();
 }
 
 /*---------------------------------------------------------------------------*/
 
-void appEndMove(int i)
+static void appEndMove(Cardinal i)
 {
   char s[0xff];
-  int j, k, l, m;
+  Cardinal j, k, l, m;
   AppRec *apps;
 
   if (aw.n_selections == 0 || aw.apps[i].selected) return;
@@ -353,7 +360,7 @@ void appEndMove(int i)
   XTFREE(aw.apps);
   aw.apps = apps;
   updateApplicationDisplay();
-  writeApplicationData(resources.app_file);
+  writeApplicationData();
 }
 
 /*---------------------------------------------------------------------------*/
@@ -366,25 +373,12 @@ static void copyApps(void)
 					  sizeof(AppRec));
   for (j=0; j<n_apps; j++)
     if (aw.apps[j].selected) {
-      Pixmap icon_bm;
-      Boolean loaded = False;
-      if (aw.apps[j].loaded)
-	if ((icon_bm = readIcon(aw.apps[j].icon,0,0)) == None) {
-	  error("Can't read icon for", aw.apps[j].name);
-	  icon_bm = defaultIcon(aw.apps[j].name, aw.apps[j].directory,
-				aw.apps[j].fname);
-	} else
-	  loaded = True;
-      else
-	icon_bm = aw.apps[j].icon_bm;
       aw.apps[aw.n_apps].name = XtNewString(aw.apps[j].name);
       aw.apps[aw.n_apps].directory = XtNewString(aw.apps[j].directory);
-      aw.apps[aw.n_apps].fname = XtNewString(aw.apps[j].fname);
       aw.apps[aw.n_apps].icon = XtNewString(aw.apps[j].icon);
       aw.apps[aw.n_apps].push_action = XtNewString(aw.apps[j].push_action);
       aw.apps[aw.n_apps].drop_action = XtNewString(aw.apps[j].drop_action);
-      aw.apps[aw.n_apps].icon_bm = icon_bm;
-      aw.apps[aw.n_apps].loaded = loaded;
+      aw.apps[aw.n_apps].icon_bm = aw.apps[j].icon_bm;
       aw.apps[aw.n_apps].form = aw.apps[aw.n_apps].toggle =
 	aw.apps[aw.n_apps].label = NULL;
       aw.n_apps++;
@@ -407,7 +401,7 @@ void appEndCopyInBox(void)
 
   copyApps();
   updateApplicationDisplay();
-  writeApplicationData(resources.app_file);
+  writeApplicationData();
 }
 
 /*---------------------------------------------------------------------------*/
@@ -435,36 +429,18 @@ void appEndCopy(int i)
   XTFREE(aw.apps);
   aw.apps = apps;
   updateApplicationDisplay();
-  writeApplicationData(resources.app_file);
+  writeApplicationData();
 }
 
 /*---------------------------------------------------------------------------*/
 
 void appEndDrag(int i)
 {
-  char **argv;
-
   if (*aw.apps[i].drop_action) {
     char *action = varPopup(aw.apps[i].icon_bm, aw.apps[i].drop_action);
     if (!action) return;
-    if (*aw.apps[i].fname) {
-      int l;
-      char path[MAXPATHLEN];
-      strcpy(path, aw.apps[i].directory);
-      if (*path)
-	fnexpand(path);
-      else
-	strcpy(path, user.home);
-      l = strlen(path);
-      if (path[l-1] != '/')
-	path[l++] = '/';
-      strcpy(path+l, aw.apps[i].fname);
-      argv = makeArgv2(action, path);
-    } else
-      argv = makeArgv(action);
-    argv = expandArgv(argv);
-    executeApplication(user.shell, move_info.fw->directory, argv);
-    freeArgv(argv);
+    fileCallWithSelected(action);
+    XTFREE(action);
   }
 }
 
@@ -474,55 +450,55 @@ void appEndDragInBox(void)
 {
   int i;
 
+  if (aw.readonly)
+	  return;
+
   for (i=0; i<move_info.fw->n_files; i++)
     if (move_info.fw->files[i]->selected) {
       if (S_ISDIR(move_info.fw->files[i]->stats.st_mode)) {
+	char *fullname = dirConcat(move_info.fw->directory,
+			move_info.fw->files[i]->name);
 	installApplication(move_info.fw->files[i]->name,
-			   move_info.fw->directory,
-			   move_info.fw->files[i]->name,
-			   move_info.fw->files[i]->type?
-			   move_info.fw->files[i]->type->icon:"",
+                           fullname,
+			   "",
 			   "OPEN",
 			   "");
+	XtFree(fullname);
       } else if (move_info.fw->files[i]->stats.st_mode &
 		 (S_IXUSR | S_IXGRP | S_IXOTH)) {
 	char *push_action, *drop_action;
 	push_action = move_info.fw->files[i]->name;
-	drop_action = (char *)alloca(strlen(push_action)+4);
+	drop_action = (char *)alloca(strlen(push_action)+6);
 	strcpy(drop_action, push_action);
-	strcat(drop_action, " $*");
+	strcat(drop_action, " \"$*\"");
 	installApplication(move_info.fw->files[i]->name,
 			   move_info.fw->directory,
 			   "",
-			   move_info.fw->files[i]->type?
-			   move_info.fw->files[i]->type->icon:"",
 			   push_action,
 			   drop_action);
-      } else if (move_info.fw->files[i]->type) {
-	installApplication(move_info.fw->files[i]->name,
-			   move_info.fw->directory,
-			   move_info.fw->files[i]->name,
-			   move_info.fw->files[i]->type->icon,
-			   move_info.fw->files[i]->type->push_action,
-			   move_info.fw->files[i]->type->drop_action);
       } else {
+// TODO: re-add support for storing how to start that file here
+	char *editcmd = spaceConcat("EDIT",
+			move_info.fw->files[i]->name);
 	installApplication(move_info.fw->files[i]->name,
 			   move_info.fw->directory,
-			   move_info.fw->files[i]->name,
 			   "",
-			   "EDIT",
+			   editcmd,
 			   "");
+	XtFree(editcmd);
       }
     }
   updateApplicationDisplay();
-  writeApplicationData(resources.app_file);
+  writeApplicationData();
 }
 
 /*---------------------------------------------------------------------------*/
 
-void appSelect(Widget w, XEvent *event, String *params, Cardinal *num_params)
+void appSelect(Widget w, UNUSED(XEvent *event),
+	       UNUSED(String *params), UNUSED(Cardinal *num_params))
 {
-  int i, j;
+  Cardinal i;
+  int j;
   Pixel back, fore;
   
   j = findAppWidget(w);
@@ -547,7 +523,8 @@ void appSelect(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
 /*---------------------------------------------------------------------------*/
 
-void appToggle(Widget w, XEvent *event, String *params, Cardinal *num_params)
+void appToggle(Widget w, UNUSED(XEvent *event),
+               UNUSED(String *params), UNUSED(Cardinal *num_params))
 {
   int i;
   Pixel pix;
@@ -569,3 +546,14 @@ void appToggle(Widget w, XEvent *event, String *params, Cardinal *num_params)
     aw.n_selections--;
 }
 
+/*---------------------------------------------------------------------------*/
+void appTrackCursor(Widget w, XEvent *event, UNUSED(String *params),
+		 UNUSED(Cardinal *num_params))
+{
+	if (dragging)
+		drag_set_dropable(XtDisplay(w), 
+				event->type != LeaveNotify && !aw.readonly);
+	else if (dragging_app)
+		awDragGrab(event->type != LeaveNotify && !aw.readonly);
+}
+
