@@ -1,11 +1,31 @@
-/*---------------------------------------------------------------------------
-  Module FmAwCb
+/*
+ *  This file is part of X-File Manager XFM
+ *  ----------------------------------------------------------------------
+  FmAwCb.c
 
   (c) S.Marlow 1990-92
   (c) A.Graef 1994
 
+  modified 2003,2005,2006 by Bernhard R. Link (see Changelog)
+
   Callback routines for widgets in the application window
----------------------------------------------------------------------------*/
+ *  ----------------------------------------------------------------------
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <xfmconfig.h>
 
 #include <stdio.h>
 #include <memory.h>
@@ -13,41 +33,44 @@
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 
+#include "global.h"
+
 #include "Am.h"
 
 /*---------------------------------------------------------------------------
   PUBLIC FUNCTIONS
 ---------------------------------------------------------------------------*/
 
-void appInstallAppCb(Widget w, FileWindowRec *fw, XtPointer call_data)
+void appInstallAppCb(UNUSED(Widget w), UNUSED(void *fw),
+                     UNUSED(void *call_data))
 {
   installNewAppPopup();
 }
 
 /*---------------------------------------------------------------------------*/
 
-void appInstallGroupCb(Widget w, FileWindowRec *fw, XtPointer call_data)
+void appInstallGroupCb(UNUSED(Widget w), UNUSED(void *client_data),
+                       UNUSED(void *call_data))
 {
   installGroupPopup();
 }
 
 /*---------------------------------------------------------------------------*/
 
-void appEditCb(Widget w, FileWindowRec *fw, XtPointer call_data)
+void appEditCb(UNUSED(Widget w), UNUSED(void *client_data),
+               UNUSED(void *call_data))
 {
   installExistingAppPopup();
 }
 
 /*---------------------------------------------------------------------------*/
 
-void appCutCb(Widget w, FileWindowRec *fw, XtPointer call_data)
+void appCutCb(UNUSED(Widget w), UNUSED(void *client_data),
+              UNUSED(void *call_data))
 {
   char s[0xff];
   FILE *fp;
-  int i, j;
-  char name[2*MAXAPPSTRINGLEN], directory[2*MAXAPPSTRINGLEN],
-    fname[2*MAXAPPSTRINGLEN], icon[2*MAXAPPSTRINGLEN],
-    push_action[2*MAXAPPSTRINGLEN], drop_action[2*MAXAPPSTRINGLEN];
+  Cardinal i, j;
 
   if (resources.confirm_moves) {
     sprintf(s, "Cutting %d item%s from", aw.n_selections,
@@ -69,19 +92,22 @@ void appCutCb(Widget w, FileWindowRec *fw, XtPointer call_data)
 	memcpy(&aw.apps[i], &aw.apps[j], sizeof(AppRec));
       i++;
     } else {
-      expand(name, aw.apps[j].name, "\\:");
-      expand(directory, aw.apps[j].directory, "\\:");
-      expand(fname, aw.apps[j].fname, "\\:");
-      expand(icon, aw.apps[j].icon, "\\:");
-      expand(push_action, aw.apps[j].push_action, "\\:");
-      expand(drop_action, aw.apps[j].drop_action, "\\:");
-      fprintf(fp, "%s:%s:%s:%s:%s:%s\n", name, directory, fname, icon,
-	      push_action, drop_action);
+      fprintexpand(fp, aw.apps[j].name, "\\:");
+      fputc(':',fp);
+      fprintexpand(fp, aw.apps[j].directory, "\\:");
+      fputc(':',fp);
+      fputc(':',fp);
+      fprintexpand(fp, aw.apps[j].icon, "\\:");
+      fputc(':',fp);
+      fprintexpand(fp, aw.apps[j].push_action, "\\:");
+      fputc(':',fp);
+      fprintexpand(fp, aw.apps[j].drop_action, "\\:");
+      fputc('\n',fp);
       freeApplicationResources(&aw.apps[j]);
     }
   aw.n_apps =i;
   updateApplicationDisplay();
-  writeApplicationData(resources.app_file);
+  writeApplicationData();
 
   if (fclose(fp))
     sysError("Error writing clip file:");
@@ -89,14 +115,12 @@ void appCutCb(Widget w, FileWindowRec *fw, XtPointer call_data)
 
 /*---------------------------------------------------------------------------*/
 
-void appCopyCb(Widget w, FileWindowRec *fw, XtPointer call_data)
+void appCopyCb(UNUSED(Widget w), UNUSED(void *client_data),
+               UNUSED(void *call_data))
 {
   char s[0xff];
   FILE *fp;
-  int i;
-  char name[2*MAXAPPSTRINGLEN], directory[2*MAXAPPSTRINGLEN],
-    fname[2*MAXAPPSTRINGLEN], icon[2*MAXAPPSTRINGLEN],
-    push_action[2*MAXAPPSTRINGLEN], drop_action[2*MAXAPPSTRINGLEN];
+  Cardinal i;
 
   if (resources.confirm_copies) {
     sprintf(s, "Copying %d item%s from", aw.n_selections,
@@ -114,14 +138,17 @@ void appCopyCb(Widget w, FileWindowRec *fw, XtPointer call_data)
 
   for (i=0; i < aw.n_apps; i++)
     if (aw.apps[i].selected) {
-      expand(name, aw.apps[i].name, "\\:");
-      expand(directory, aw.apps[i].directory, "\\:");
-      expand(fname, aw.apps[i].fname, "\\:");
-      expand(icon, aw.apps[i].icon, "\\:");
-      expand(push_action, aw.apps[i].push_action, "\\:");
-      expand(drop_action, aw.apps[i].drop_action, "\\:");
-      fprintf(fp, "%s:%s:%s:%s:%s:%s\n", name, directory, fname, icon,
-	      push_action, drop_action);
+      fprintexpand(fp, aw.apps[i].name, "\\:");
+      fputc(':',fp);
+      fprintexpand(fp, aw.apps[i].directory, "\\:");
+      fputc(':',fp);
+      fputc(':',fp);
+      fprintexpand(fp, aw.apps[i].icon, "\\:");
+      fputc(':',fp);
+      fprintexpand(fp, aw.apps[i].push_action, "\\:");
+      fputc(':',fp);
+      fprintexpand(fp, aw.apps[i].drop_action, "\\:");
+      fputc('\n',fp);
     }
   
   if (fclose(fp))
@@ -130,41 +157,17 @@ void appCopyCb(Widget w, FileWindowRec *fw, XtPointer call_data)
 
 /*---------------------------------------------------------------------------*/
 
-void appPasteCb(Widget w, FileWindowRec *fw, XtPointer call_data)
+void appPasteCb(UNUSED(Widget w), UNUSED(void *client_data),
+                UNUSED(void *call_data))
 {
   FILE *fp;
-  char *name, *directory, *fname, *icon, *push_action, *drop_action;
-  char s[MAXAPPSTRINGLEN];
   int p;
   
   if (!(fp = fopen(resources.app_clip, "r"))) return;
 
-  for (; (p = parseApp(fp, &name, &directory, &fname, &icon, &push_action,
-		       &drop_action)) > 0; aw.n_apps++) {
-    aw.apps = (AppList) XTREALLOC(aw.apps, (aw.n_apps+1)*sizeof(AppRec) );
-    aw.apps[aw.n_apps].name = XtNewString(strparse(s, name, "\\:"));
-    aw.apps[aw.n_apps].directory = XtNewString(strparse(s, directory, "\\:"));
-    aw.apps[aw.n_apps].fname = XtNewString(strparse(s, fname, "\\:"));
-    aw.apps[aw.n_apps].icon = XtNewString(strparse(s, icon, "\\:"));
-    aw.apps[aw.n_apps].push_action = XtNewString(strparse(s, push_action,
-							  "\\:"));
-    aw.apps[aw.n_apps].drop_action = XtNewString(strparse(s, drop_action,
-							  "\\:"));
-    aw.apps[aw.n_apps].loaded = False;
-    if (!aw.apps[aw.n_apps].icon[0])
-      aw.apps[aw.n_apps].icon_bm = defaultIcon(aw.apps[aw.n_apps].name,
-					       aw.apps[aw.n_apps].directory,
-					       aw.apps[aw.n_apps].fname);
-    else if ((aw.apps[aw.n_apps].icon_bm = readIcon(aw.apps[aw.n_apps].icon,0,0))
-	     == None) {
-      fprintf(stderr, "%s: can't read icon for application %s\n",
-	      progname, aw.apps[aw.n_apps].name);
-      aw.apps[aw.n_apps].icon_bm = defaultIcon(aw.apps[aw.n_apps].name,
-					       aw.apps[aw.n_apps].directory,
-					       aw.apps[aw.n_apps].fname);
-    } else
-      aw.apps[aw.n_apps].loaded = True;
-  }
+  do{
+	  p = parseApp(fp,True);
+  } while( p > 0 );
 
   if (p == -1)
     error("Error in clip file", "");
@@ -173,15 +176,16 @@ void appPasteCb(Widget w, FileWindowRec *fw, XtPointer call_data)
     sysError("Error reading clip file:");
 
   updateApplicationDisplay();
-  writeApplicationData(resources.app_file);
+  writeApplicationData();
 }
 
 /*---------------------------------------------------------------------------*/
 
-void appRemoveCb(Widget w, FileWindowRec *fw, XtPointer call_data)
+void appRemoveCb(UNUSED(Widget w), UNUSED(void *client_data),
+                 UNUSED(void *call_data))
 {
   char s[0xff];
-  int i, j;
+  Cardinal i, j;
 
   if (resources.confirm_deletes) {
     sprintf(s, "Deleting %d item%s from", aw.n_selections,
@@ -199,14 +203,15 @@ void appRemoveCb(Widget w, FileWindowRec *fw, XtPointer call_data)
       freeApplicationResources(&aw.apps[j]);
   aw.n_apps =i;
   updateApplicationDisplay();
-  writeApplicationData(resources.app_file);
+  writeApplicationData();
 }
 
 /*---------------------------------------------------------------------------*/
 
-void appSelectAllCb(Widget w, FileWindowRec *fw, XtPointer call_data)
+void appSelectAllCb(UNUSED(Widget w), UNUSED(void *client_data),
+                    UNUSED(void *call_data))
 {
-  int i;
+  Cardinal i;
   Pixel pix;
   
   for (i=0; i < aw.n_apps; i++)
@@ -220,9 +225,10 @@ void appSelectAllCb(Widget w, FileWindowRec *fw, XtPointer call_data)
 
 /*---------------------------------------------------------------------------*/
 
-void appDeselectCb(Widget w, FileWindowRec *fw, XtPointer call_data)
+void appDeselectCb(UNUSED(Widget w), UNUSED(void *client_data),
+                   UNUSED(void *call_data))
 {
-  int i;
+  Cardinal i;
   Pixel pix;
   
   for (i=0; i < aw.n_apps; i++)
@@ -236,64 +242,65 @@ void appDeselectCb(Widget w, FileWindowRec *fw, XtPointer call_data)
 
 /*---------------------------------------------------------------------------*/
 
-void appLoadCb(Widget w, FileWindowRec *fw, XtPointer call_data)
+void appLoadCb(UNUSED(Widget w), UNUSED(void *client_data),
+               UNUSED(void *call_data))
 {
-  int i;
-
-  for(i=0; i<aw.n_apps; i++)
-    freeApplicationResources(&aw.apps[i]);
-  XTFREE(aw.apps);
-
-  readApplicationData(resources.app_file);
-  updateApplicationDisplay();
+  AmReadUpdate();
 }
 
 /*---------------------------------------------------------------------------*/
 
-void appMainCb(Widget w, FileWindowRec *fw, XtPointer call_data)
+void appMainCb(UNUSED(Widget w), UNUSED(void *client_data),
+               UNUSED(void *call_data))
 {
-  strcpy(resources.app_file, resources.main_app_file);
+	XTFREE(aw.app_file);
+#ifdef SYSTEMWIDE_DEFAULTS
+	if (!exists(resources.main_app_file))
+		/* if the main_app_file is not found, use the system_main_app_file
+		 * instead. (Only change app_file and not main_app_file, to
+		 * allow adding a user-file while running). */
+		aw.app_file = XtNewString(resources.system_main_app_file);
+	else
+#endif
+		aw.app_file = XtNewString(resources.main_app_file);
   clearApplicationsStack();
-  appLoadCb(w, fw, call_data);
+  AmReadUpdate();
 }
 
 /*---------------------------------------------------------------------------*/
 
-void appBackCb(Widget w, FileWindowRec *fw, XtPointer call_data)
+void appBackCb(UNUSED(Widget w), UNUSED(void *client_data),
+               UNUSED(void *call_data))
 {
-  int i;
-
   if (n_appst > 0) {
-    for(i=0; i<aw.n_apps; i++)
-      freeApplicationResources(&aw.apps[i]);
-    XTFREE(aw.apps);
-
     popApplicationsFile();
-    readApplicationData(resources.app_file);
-    updateApplicationDisplay();
+    AmReadUpdate();
   }
 }
 
 /*---------------------------------------------------------------------------*/
 
-void appOpenCb(Widget w, FileWindowRec *fw, XtPointer call_data)
+void appOpenCb(UNUSED(Widget w), UNUSED(void *client_data),
+               UNUSED(void *call_data))
 {
   newFileWindow(user.home,resources.initial_display_type,False);
 }
 
 /*---------------------------------------------------------------------------*/
 
-void appCloseCb(Widget w, FileWindowRec *fw, XtPointer call_data)
+void appCloseCb(UNUSED(Widget w), UNUSED(void *client_data),
+		UNUSED(void *call_data))
 {
   if (resources.confirm_quit && !confirm("", "Exit file manager?", ""))
     return;
 
-  quit();
+  quit(0);
 }
 
 /*---------------------------------------------------------------------------*/
 
-void aboutCb(Widget w, FileWindowRec *fw, XtPointer call_data)
+void aboutCb(UNUSED(Widget w), UNUSED(void *client_data),
+             UNUSED(void *call_data))
 {
   aboutPopup();
 }
