@@ -1,31 +1,49 @@
-/*-----------------------------------------------------------------------------
-  Module FmErrors
+/*
+ *  This file is part of X-File Manager XFM
+ *  ----------------------------------------------------------------------
+  FmErrors.c
   
   (c) Simon Marlow 1990-1993
   (c) Albert Graef 1994
 
+  modified 2005,2006 by Bernhard R. Link (see Changelog)
+
   Error handling routines
------------------------------------------------------------------------------*/
+ *  ----------------------------------------------------------------------
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <xfmconfig.h>
 
 #include <errno.h>
+#include <stdlib.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/Label.h>
-#include <X11/Xaw/Box.h>
+#include <X11/Xaw3d/Form.h>
+#include <X11/Xaw3d/Label.h>
+#include <X11/Xaw3d/Box.h>
+
+#include "global.h"
 
 #include "Am.h"
 #include "Fm.h"
 
 #define LABEL_WIDTH 300
 
-/* Uggh ...... */
-#ifndef linux
-extern char *sys_errlist[];
-#endif
-
 /*-----------------------------------------------------------------------------
   STAIC DATA
 -----------------------------------------------------------------------------*/
@@ -84,7 +102,8 @@ static Arg button_box_args[] = {
   PRIVATE FUNCTIONS
 -----------------------------------------------------------------------------*/
 
-static void errorOkCb(Widget w, XtPointer client_data, XtPointer call_data)
+static void errorOkCb(UNUSED(Widget w), UNUSED(void *client_data),
+                      UNUSED(void *call_data))
 {
   XtPopdown(errors.shell);
   error_flag = True;
@@ -94,15 +113,15 @@ static void errorOkCb(Widget w, XtPointer client_data, XtPointer call_data)
   Button Data
 -----------------------------------------------------------------------------*/
 
-static ButtonRec error_buttons[] = {
-  { "cancel", "Cancel", (FmCallbackProc *) errorOkCb }
+static const ButtonRec error_buttons[] = {
+  { "cancel", "Cancel", errorOkCb }
 };
 
 /*-----------------------------------------------------------------------------
   PUBLIC FUNCTIONS
 -----------------------------------------------------------------------------*/
 
-void createErrorPopup()
+void createErrorPopup(void)
 {
   Widget form, bitmap, button_box;
 
@@ -148,7 +167,7 @@ void createErrorPopup()
 
 /*---------------------------------------------------------------------------*/
 
-void untilOk()
+static void untilOk(void)
 {
   XEvent e;
 
@@ -162,19 +181,19 @@ void untilOk()
 
 /*---------------------------------------------------------------------------*/
 
-void sysError(String string1)
+void sysError(const char *string1)
 {
   if (!error_flag) /* recursive errors are possible - we just ignore them */
     return;
   XtVaSetValues(errors.label1, XtNlabel, string1, NULL);
-  XtVaSetValues(errors.label2, XtNlabel, sys_errlist[errno], NULL);
+  XtVaSetValues(errors.label2, XtNlabel, strerror(errno), NULL);
   popupByCursor(errors.shell, XtGrabExclusive);
   untilOk();
 }
 
 /*---------------------------------------------------------------------------*/
 
-void error(String string1, String string2)
+void error(const char *string1, const char *string2)
 {
   if (!error_flag)
     return;
@@ -186,7 +205,7 @@ void error(String string1, String string2)
 
 /*---------------------------------------------------------------------------*/
 
-void abortXfm(String message)
+void abortXfm(const char *message)
 {
   fprintf(stderr,"%s: %s -- abort\n", progname, message);
   exit(1);
