@@ -1,47 +1,77 @@
-/*-----------------------------------------------------------------------------
-  Module FmFw.c                                                             
+/*
+ *  This file is part of X-File Manager XFM
+ *  ----------------------------------------------------------------------
+  FmFw.c                                                             
 
   (c) Simon Marlow 1991
   (c) Albert Graef 1994
-                                                                           
-  functions & data for creating a file window, and various functions        
-  related to file windows                                                   
 
   modified 1-29-95 by rodgers@lvs-emh.lvs.loral.com (Kevin M. Rodgers)
   to add filtering of icon/text directory displays by a filename filter.
 
   modified 7-1997 by strauman@sun6hft.ee.tu-berlin.de to add
-  different enhancements (see README-NEW).
+  different enhancements (see README-1.4).
+                                                                           
+  modified 2005,2006 by Bernhard R. Link (see Changelog)
 
------------------------------------------------------------------------------*/
+  functions & data for creating a file window, and various functions        
+  related to file windows                                                   
+ *  ----------------------------------------------------------------------
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <xfmconfig.h>
+
+#include <assert.h>
 #include <pwd.h>
 #include <time.h>
 #include <string.h>
+#include <stdlib.h>
+#include <stdbool.h>
 #include <ctype.h>
 
 #ifdef _AIX
 #include <sys/resource.h>
 #endif
+#ifdef HAVE_FSTAB_H
+#include <fstab.h>
+#endif
+
+#define VIEWPORT_HACK 1
 
 #include <sys/wait.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/Shell.h>
 #include <X11/StringDefs.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/Box.h>
-#include <X11/Xaw/Viewport.h>
-#include <X11/Xaw/Toggle.h>
-#include <X11/Xaw/Label.h>
+#include <X11/Xaw3d/Form.h>
+#include <X11/Xaw3d/Box.h>
+#include <X11/Xaw3d/Viewport.h>
+#include <X11/Xaw3d/Toggle.h>
+#include <X11/Xaw3d/Label.h>
 
 #include "XtHelper.h"
 
+#include "global.h"
+
 #include "TextFileList.h"
 #include "IconFileList.h"
 
 #include "Am.h"
 #include "Fm.h"
+#include "execute.h"
 
 #ifdef ENHANCE_HISTORY
 #include "FmHistory.h"
@@ -64,6 +94,8 @@
 #include "FmLog.h"
 #endif
 
+#include "mime.h"
+
 #define FW_WIDTH 400
 #define FW_HEIGHT 300
 #define TEXT_PADDING 10
@@ -78,21 +110,15 @@ extern HistoryList path_history;
 #endif
 
 FileWindowList file_windows = NULL;
-FileWindowRec *popup_fw;
-Widget file_popup_widget, *file_popup_items;
-Widget dir_popup_widget, *dir_popup_items;
 
-int n_types;
-TypeList types;
-
-int n_devices;
+Cardinal n_devices;
 DevList devs;
 
 /*-----------------------------------------------------------------------------
   STATIC DATA                                       
 -----------------------------------------------------------------------------*/
 
-static MenuItemRec file_popup_menu[] = {
+static const FmMenuItemRec file_popup_menu[] = {
   { "edit", "Edit", fileEditCb },
   { "view", "View", fileViewCb },
   { "line1", NULL, NULL },
@@ -108,7 +134,7 @@ static MenuItemRec file_popup_menu[] = {
   { "chmod", "Permissions...", chmodPopup }
 };
 
-static MenuItemRec dir_popup_menu[] = {
+static const FmMenuItemRec dir_popup_menu[] = {
   { "open", "Open", fileOpenCb },
   { "line1", NULL, NULL },
   { "rename", "Rename...", renamePopup },
@@ -123,7 +149,7 @@ static MenuItemRec dir_popup_menu[] = {
   { "chmod", "Permissions...", chmodPopup }
 };
 
-static MenuItemRec file_menu[] = {
+static const FmMenuItemRec file_menu[] = {
   { "new", "New...", createFilePopup },
   { "line1", NULL, NULL },
   { "move", "Move...", movePopup },
@@ -141,12 +167,12 @@ static MenuItemRec file_menu[] = {
   { "line4", NULL, NULL },
   { "xterm", "Xterm", xtermCb },
   { "line5", NULL, NULL },
-  { "about", "About xfm...", aboutCb },
+  { "about", "About xfm...", fileAboutCb },
   { "line6", NULL, NULL },
-  { "quit", "Quit", appCloseCb },
+  { "quit", "Quit", fileQuitCb },
 };
 
-static MenuItemRec folder_menu[] = {
+static const FmMenuItemRec folder_menu[] = {
   { "new", "New...", mkdirPopup },
   { "line1", NULL, NULL },
   { "goto", "Go to...", goToPopup },
@@ -161,7 +187,7 @@ static MenuItemRec folder_menu[] = {
   { "close", "Close", fileCloseCb },
 };
 
-static MenuItemRec view_menu[] = {
+static const FmMenuItemRec view_menu[] = {
   { "tree", "Tree",  fileTreeCb },
   { "icons", "Icons",  fileIconsCb },
   { "text", "Text",  fileTextCb },
@@ -185,7 +211,7 @@ static MenuItemRec view_menu[] = {
 /* Don't change the order of menu entries
  * (showTxtOptsCb() and updateTxtOpts() depend on it)
  */
-static MenuItemRec text_opts_menu[]={
+static const FmMenuItemRec text_opts_menu[]={
   { "show Inode", "Show Inode", showTxtOptsCb},
   { "show Type", "Show Type", showTxtOptsCb},
   { "show Permission", "Show Permission", showTxtOptsCb},
@@ -237,7 +263,8 @@ static Arg viewport_args[] = {
   { XtNbottom, XtChainBottom },
   { XtNleft, XtChainLeft },
   { XtNright, XtChainRight },
-  { XtNallowVert, (XtArgVal) True }
+  { XtNallowVert, (XtArgVal) True },
+  { XtNallowHoriz, (XtArgVal) True }
 };
 
 static Arg status_args[] = {
@@ -361,34 +388,13 @@ static char tree_translations_s[] = "\
   <Btn1Down>,<Leave>  : fileBeginDrag(1,move)\n\
   <Btn2Down>,<Btn2Up> : fileToggle()\n\
   <Btn2Down>,<Leave>  : fileBeginDrag(2,copy)\n\
-  <Btn3Down>          : dirPopup()\n";
-
-static char file_list_translations_s[] = "\
-  <Enter>	      : dispatchDirExeFile(fileHighlight,fileHighlight,fileMaybeHighlight,\"\")\n\
-  <Leave>	      : dispatchDirExeFile(resetCursor,resetCursor,unhighlight,\"\")\n\
-  <Btn1Up>(2)         : dispatchDirExeFile(fileOpenDir,fileExecFile,fileExecAction,\"\")\n\
-  <Btn1Down>,<Btn1Up> : fileSelect()\n\
-  <Btn1Down>,<Motion> : fileBeginDrag(1,move)\n\
-  <Btn2Down>,<Btn2Up> : fileToggle()\n\
-  <Btn2Down>,<Motion> : fileBeginDrag(2,copy)\n\
-  <Btn3Down>          : dispatchDirExeFilePopup(dirPopup,filePopup,filePopup,\"\")\n\
-	<Key>Delete   : notify(*file*delete)\n\
-	<Key>BackSpace: notify(*file*delete)\n\
-	<Key>m	      : notify(*file*move)\n\
-	<Key>n	      : notify(*file*new)\n\
-	<Key>x	      : notify(*file*xterm)\n\
-	<Key>q	      : notify(*file*quit)\n\
-	<Key>c	      : notify(*file*copy)\n\
-	<Key>s	      : notify(*file*select)\n\
-	<Key>o	      : notify(\"*file*own selection\")\n\
-	<Key>a	      : notify(\"*file*select all\")\n\
-	<Key>u	      : notify(\"*file*deselect all\")\n\
-	<Key>l	      : notify(*file*link)\n";
+  Any<Btn3Down>       : dirPopup()\n";
 
 #endif
   
-static void dummy(Widget w, XEvent *event, String *params, 
-		       Cardinal *num_params) {}
+static void dummy(UNUSED(Widget w), UNUSED(XEvent *event), 
+		  UNUSED(String *params), UNUSED(Cardinal *num_params)) {
+}
 
 
 static XtActionsRec file_actions[] = {
@@ -398,7 +404,6 @@ static XtActionsRec file_actions[] = {
   { "fileHighlight", fileHighlight },
   { "fileOpenDir", fileOpenDir },
   { "fileBeginDrag", fileBeginDrag },
-  { "fileExecFile", fileExecFile },
   { "fileExecAction", fileExecAction },
   { "resetCursor", resetCursor },
   { "trackCursor", trackCursor },
@@ -429,7 +434,7 @@ static int longestName(FileWindowRec *fw)
 }
 
 static void
-unmapTxtOptsCb(Widget w, XtPointer cld, XtPointer cad)
+unmapTxtOptsCb(Widget w, XtPointer cld, UNUSED(XtPointer cad))
 {
  FileWindowRec	*fw=(FileWindowRec *)cld;
  Widget			options_button;
@@ -488,202 +493,8 @@ static void fileListCb(Widget w, XtPointer cld, XtPointer cad)
 
 /*---------------------------------------------------------------------------*/
 
-static int parseType(FILE *fp, char **pattern,
-#ifdef MAGIC_HEADERS
-                     char **magic_type,
-#endif
-                     char **icon, char **push_action, char **drop_action)
-{
-  static char s[MAXCFGLINELEN];
-  int l;
-
- start:
-  if (feof(fp)||!fgets(s, MAXCFGLINELEN, fp))
-    return 0;
-  l = strlen(s);
-  if (s[l-1] == '\n')
-    s[--l] = '\0';
-  if (!l || *s == '#')
-    goto start;
-  if (!(*pattern = split(s, ':')))
-    return -1;
-#ifdef MAGIC_HEADERS
-  if (**pattern == '<') {
-    char *ptr;
-    ptr = *pattern + 1;
-    while(*ptr && (*ptr != '>' || ptr[-1] == '\\'))
-	ptr++;
-    if(*ptr != '>')
-	return -1;
-    *ptr = '\0';
-    *magic_type = *pattern + 1;
-    *pattern = ptr + 1;
-  }
-  else
-    *magic_type = NULL;
-#endif
-  if (!(*icon = split(NULL, ':')))
-    return -1;
-  if (!(*push_action = split(NULL, ':')))
-    return -1;
-  if (!(*drop_action = split(NULL, ':')))
-    return -1;
-  return l;
-}
-
-/*---------------------------------------------------------------------------*/
-
-static void readFileBitmaps(void)
-{
-  int i;
-
-  for (i=0; i<n_types; i++)
-    if (!types[i].icon[0]) {
-      types[i].icon_bm = bm[FILE_BM];
-      types[i].bm_width=builtin_types[FILE_T].bm_width;
-      types[i].bm_height=builtin_types[FILE_T].bm_height;
-    }
-    else if ((types[i].icon_bm = readIcon(types[i].icon,
-					&types[i].bm_width,
-					&types[i].bm_height)) == None) {
-#ifdef MAGIC_HEADERS
-      fprintf(stderr, "%s: can't read icon for type %s%s%s%s%s%s\n", progname,
-	      types[i].magic_type?"<":"",
-	      types[i].magic_type?types[i].magic_type:"",
-	      types[i].magic_type?">":"",
-	      types[i].dir<0?"*":"", types[i].pattern,
-	      types[i].dir>0?"*":"");
-#else
-      fprintf(stderr, "%s: can't read icon for type %s%s%s\n", progname,
-	      types[i].dir<0?"*":"", types[i].pattern,
-	      types[i].dir>0?"*":"");
-#ifndef ENHANCE_BUGFIX
-      types[i].icon_bm = bm[FILE_BM];
-#endif
-#endif
-#ifdef ENHANCE_BUGFIX  
-     /* we need a default icon in any case, whether we use MAGIC_HEADERS or not */
-      types[i].icon_bm = bm[FILE_BM];
-#endif
-      types[i].bm_width=builtin_types[FILE_T].bm_width;
-      types[i].bm_height=builtin_types[FILE_T].bm_height;
-    }
-}
-
-/*---------------------------------------------------------------------------*/
-
-static void readFileTypes(String path)
-{
-  FILE *fp;
-  char *pattern, *icon, *push_action, *drop_action;
-#ifdef MAGIC_HEADERS
-  char *magic_type;
-#endif
-  char s[MAXCFGSTRINGLEN];
-  int i, l, p;
-  
-  n_types = 0;
-  types = NULL;
-  
-  if (!(fp = fopen(path, "r"))) return;
-
-  for (i=0; (p = parseType(fp, &pattern,
-#ifdef MAGIC_HEADERS
-                           &magic_type,
-#endif
-                           &icon, &push_action,
-			   &drop_action)) > 0; i++) {
-    types = (TypeList) XTREALLOC(types, (i+1)*sizeof(TypeRec) );
-    l = strlen(pattern);
-    if (pattern[0] == '*') {
-      types[i].dir = -1;
-#ifdef MAGIC_HEADERS
-      strparse(s, pattern+1, "\\:<>");
-#else
-      strparse(s, pattern+1, "\\:");
-#endif
-    } else if (pattern[l-1] == '*') {
-      types[i].dir = 1;
-      pattern[l-1] = '\0';
-#ifdef MAGIC_HEADERS
-      strparse(s, pattern, "\\:<>");
-#else
-      strparse(s, pattern, "\\:");
-#endif
-    } else {
-      types[i].dir = 0;
-#ifdef MAGIC_HEADERS
-      strparse(s, pattern, "\\:<>");
-#else
-      strparse(s, pattern, "\\:");
-#endif
-    }
-    types[i].len = strlen(s);
-    types[i].pattern = XtNewString(s);
-#ifdef MAGIC_HEADERS
-    if(magic_type)
-      types[i].magic_type = XtNewString(strparse(s, magic_type, "\\:"));
-    else
-      types[i].magic_type = NULL;
-#endif
-    types[i].icon = XtNewString(strparse(s, icon, "\\:"));
-    types[i].push_action = XtNewString(strparse(s, push_action, "\\:"));
-    types[i].drop_action = XtNewString(strparse(s, drop_action, "\\:"));
-  }
-
-  if (p == -1)
-    error("Error in configuration file", "");
-
-  n_types = i;
-  
-  if (fclose(fp))
-    sysError("Error reading configuration file:");
-
-  readFileBitmaps();
-}
-
-/*---------------------------------------------------------------------------*/
-
-#ifdef MAGIC_HEADERS
-TypeRec *fileType(char *name, char *magic_type)
-#else
-TypeRec *fileType(char *name)
-#endif
-{
-  int i, l = strlen(name);
-
-  for (i = 0; i < n_types; i++) {
-#ifdef MAGIC_HEADERS
-    if (types[i].magic_type) {
-      if(strcmp(types[i].magic_type, magic_type))
-        continue;
-      else if (!strcmp(types[i].pattern, "")) /* Empty pattern. */
-        return types+i;
-    }
-#endif
-    switch (types[i].dir) {
-    case 0:
-      if (!strcmp(name, types[i].pattern))
-	return types+i;
-      break;
-    case 1:
-      if (!strncmp(types[i].pattern, name, types[i].len))
-	return types+i;
-      break;
-    case -1:
-      if (l >= types[i].len && !strncmp(types[i].pattern, name+l-types[i].len,
-					types[i].len))
-	return types+i;
-      break;
-    }
-  }
-  return NULL;
-}
-
-/*---------------------------------------------------------------------------*/
-
 static int parseDev(FILE *fp, char **name, char **mount_action,
-		    char **umount_action)
+		    char **umount_action, char **type)
 {
   static char s[MAXCFGLINELEN];
   int l;
@@ -702,30 +513,103 @@ static int parseDev(FILE *fp, char **name, char **mount_action,
     return -1;
   if (!(*umount_action = split(NULL, ':')))
     return -1;
+  *type = split(NULL, ':');
   return l;
 }
 
 /*---------------------------------------------------------------------------*/
+#ifdef HAVE_FSTAB_H
+static char *getFSIcon(const char *fstype, const char *translations) {
+	size_t fslen = strlen(fstype);
+	const char *rule = translations;
+	const char *result = translations;
+
+	while ((rule = strchr(rule, ',')) != NULL) {
+		rule++;
+		if (strncmp(rule,fstype,fslen) != 0 || rule[fslen] != '=')
+			continue;
+		result = rule + fslen + 1;
+		break;
+	}
+	rule = strchr(result, ',');
+	if (rule == NULL) {
+		return XtNewString(result);
+	} else {
+		char *r = XtMalloc((rule-result)+1);
+		memcpy(r, result, rule-result);
+		r[rule-result] = '\0';
+		return r;
+	}
+}
+#endif
 
 static void readDevices(String path)
 {
   FILE *fp;
-  char *name, *mount_action, *umount_action;
-  char s[MAXCFGSTRINGLEN];
-  int i, p;
+  char *name, *mount_action, *umount_action, *type;
+  char s1[MAXCFGSTRINGLEN];
+  char s2[MAXCFGSTRINGLEN];
+  char s3[MAXCFGSTRINGLEN];
+  char s4[MAXCFGSTRINGLEN];
+  int i, j, p;
   
   n_devices = 0;
   devs = NULL;
   
   if (!(fp = fopen(path, "r"))) return;
 
-  for (i=0; (p = parseDev(fp, &name, &mount_action, &umount_action)) > 0;
-       i++) {
+  i = 0;
+  while ((p = parseDev(fp, &name, &mount_action, &umount_action, &type)) > 0) {
+    name = strparse(s1, name, "\\:");
+    mount_action = strparse(s2, mount_action, "\\:");
+    umount_action = strparse(s3, umount_action, "\\:");
+    if (type != NULL)
+	    type = strparse(s4, type, "\\:");
+    else
+	    type = strcpy(s4,"disk.xpm");
+    if (strcmp(name,"GETFSENT") == 0) {
+#ifdef HAVE_FSTAB_H
+	    struct fstab *fs;
+
+	    if (setfsent() != 1 ) {
+		    sysError("setfsent() returned an error, unable to get mountable devices!\n");
+		    continue;
+	    }
+	    while ((fs = getfsent()) != NULL) {
+		    /* look for "user" or "users" option */
+		    if (strncmp(fs->fs_mntops,"user",4) != 0 && 
+				    strstr(fs->fs_mntops,",user") == NULL) {
+			    continue;
+		    }
+		    /* skip entries already in the list */
+		    for (j=0; j < i ; j++)
+			    if (strcmp(devs[j].name,fs->fs_file) == 0)
+				    break;
+		    if (j < i)
+			    continue;
+		    devs = (DevList) XTREALLOC(devs, (i+1)*sizeof(DevRec) );
+		    devs[i].name = XtNewString(fs->fs_file);
+		    devs[i].type = getFSIcon(fs->fs_vfstype,type);
+		    devs[i].mount_action = spaceConcat(mount_action,
+				    fs->fs_file);
+		    devs[i].umount_action = spaceConcat(umount_action,
+				    fs->fs_file);
+		    devs[i].mounted = 0;
+		    i++;
+	    }
+	    endfsent();
+#else
+	    sysError("No getfsent support compiled in, ignoring GETFSENT!\n");
+#endif
+	    continue;
+    }
     devs = (DevList) XTREALLOC(devs, (i+1)*sizeof(DevRec) );
-    devs[i].name = XtNewString(strparse(s, name, "\\:"));
-    devs[i].mount_action = XtNewString(strparse(s, mount_action, "\\:"));
-    devs[i].umount_action = XtNewString(strparse(s, umount_action, "\\:"));
+    devs[i].name = XtNewString(name);
+    devs[i].mount_action = XtNewString(mount_action);
+    devs[i].umount_action = XtNewString(umount_action);
+    devs[i].type = XtNewString(type);
     devs[i].mounted = 0;
+    i++;
   }
 
   if (p == -1)
@@ -739,31 +623,17 @@ static void readDevices(String path)
 
 /*---------------------------------------------------------------------------*/
 
-static int devAction(int d, char *action)
+static bool devAction(const char *action)
 {
-  int pid, status;
+  const char **command;
+  bool successfull;
 
-  if ((pid = fork()) == -1) {
-    sysError("Can't fork:");
-    return 0;
-  } else if (chdir(user.home)) {
-    sysError("Can't chdir:");
-    return 0;
-  } else if (!pid) {
-    if (resources.echo_actions)
-      fprintf(stderr, "%s\n", action);
-    freopen("/dev/null", "r", stdin);
-    if (user.arg0flag)
-      execlp(user.shell, user.shell, "-c", action, user.shell, NULL);
-    else
-      execlp(user.shell, user.shell, "-c", action, NULL);
-    perror("Exec failed");
-    exit(1);
-  } else if (waitpid(pid, &status, 0) == -1 || !WIFEXITED(status) ||
-	     WEXITSTATUS(status))
-    return 0;
-  else
-    return 1;
+  command = makeShellArgs(action, NULL);
+
+  successfull = execute_and_wait(command[0], "/", command);
+  XTFREE(command);
+
+  return successfull?1:0;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -772,7 +642,6 @@ static int devAction(int d, char *action)
 static void createFileIcons(FileWindowRec *fw)
 {
  Dimension width;
- int	  nhoriz;
 
 
 
@@ -797,8 +666,8 @@ static void createFileIcons(FileWindowRec *fw)
 
  XtAddCallback(fw->icon_box,XtNcallback,fileListCb,(XtPointer)fw);
 
- XtVaGetValues(fw->icon_box,XtNnHoriz,&nhoriz,0);
- XtVaSetValues(fw->viewport, XtNallowHoriz, nhoriz!=0 , NULL);
+ XtVaSetValues(fw->viewport, XtNallowVert, (XtArgVal)True , NULL);
+ XtVaSetValues(fw->viewport, XtNallowHoriz, (XtArgVal)True , NULL);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -857,13 +726,11 @@ static Widget createDirIcon(FileWindowRec *fw, int i, Widget horiz,Widget vert)
   Pixel back;
   Pixmap icon = None;
 
-#ifdef MAGIC_HEADERS
-  file->type = fileType(file->name, file->magic_type);
-  if (file->type)
-    icon = (XtArgVal) file->type->icon_bm;
-#endif
-  if (icon == None)
-    icon = bm[DIR_BM];
+  if( file->mime_type ) {
+	  assert( file->mime_type->data != NULL );
+	  icon = file->mime_type->data->icon_bm;
+	  assert( icon != None );
+  }
 
   /* create form */
   tree_form_args[0].value = (XtArgVal) horiz;
@@ -970,7 +837,7 @@ static void createTreeDisplay(FileWindowRec *fw)
   arrow_args[0].value = (XtArgVal) NULL;
   arrow_args[1].value = (XtArgVal) NULL;
   if (!permission(&files[1]->stats, P_EXECUTE)) {
-    arrow_args[2].value = bm[NOENTRY_CBM];
+    arrow_args[2].value = bm[NOENTRY_BM];
     arrow_args[3].value = False;
   }
   else {
@@ -1012,10 +879,10 @@ static void createTreeDisplay(FileWindowRec *fw)
     arrow_args[0].value = (XtArgVal) horiz;
     arrow_args[1].value = (XtArgVal) vert;
     if (!permission(&files[i]->stats, P_EXECUTE)) {
-      arrow_args[2].value = bm[NOENTRY_CBM];
+      arrow_args[2].value = bm[NOENTRY_BM];
       arrow_args[3].value = False;
     }
-    else if (files[i]->sym_link) {
+    else if (files[i]->info.symlink) {
       arrow_args[2].value = bm[WAVY_BM];
       arrow_args[3].value = True;
     }
@@ -1030,10 +897,10 @@ static void createTreeDisplay(FileWindowRec *fw)
   }
 #ifdef VIEWPORT_HACK
   XawFormDoLayout(fw->icon_box,False);
-  XtVaGetValues(fw->icon_box,XtNchildren,&children,XtNnumChildren,&num_children,0);
+  XtVaGetValues(fw->icon_box,XtNchildren,&children,XtNnumChildren,&num_children,(char*)0);
   XtManageChildren(children,num_children);
   if (!force_bars)
-    XtVaSetValues(fw->viewport,XtNforceBars,False,0);
+    XtVaSetValues(fw->viewport,XtNforceBars,False,(char*)0);
 #endif
 }
 
@@ -1044,7 +911,7 @@ static void createTreeDisplay(FileWindowRec *fw)
 /* find the device for a directory */
 int findDev(char *path)
 {
-  int d;
+  unsigned int d;
 
   for (d = 0; d < n_devices; d++)
     if (prefix(devs[d].name, path))
@@ -1054,31 +921,102 @@ int findDev(char *path)
 
 /*---------------------------------------------------------------------------*/
 /* mount a device */
-void mountDev(int d)
+Boolean mountDev(int d)
 {
-  if (d == -1)
-    ;
-  else if (devs[d].mounted)
-    devs[d].mounted++;
-  else
-    devs[d].mounted += devAction(d, devs[d].mount_action);
+	if (d == -1)
+		return False;
+	if (devs[d].mounted || devAction(devs[d].mount_action)) {
+		devs[d].mounted++;
+		return True;
+	}
+	return False;
 }
 
 /*---------------------------------------------------------------------------*/
 /* unmount a device */
 void umountDev(int d)
 {
-  if (d == -1 || !devs[d].mounted)
-    ;
-  else if (devs[d].mounted > 1)
-    devs[d].mounted--;
-  else
-    devs[d].mounted -= devAction(d, devs[d].umount_action);
+	if (d == -1 || devs[d].mounted <= 0)
+		return;
+	if (devs[d].mounted > 1 || devAction(devs[d].umount_action))
+		devs[d].mounted--;
+}
+
+static void loadtypeicon(struct mime_filetype *type) {
+	struct filetype_data *data;
+	Cardinal w,h;
+	Pixmap p;
+
+	if( type == NULL )
+		abortXfm("Out of Memory");
+	if( type->data == NULL ) {
+		data = (struct filetype_data*)XtCalloc(1,sizeof(struct filetype_data));
+		type->data = data;
+	} else  {
+		data = type->data;
+		if( !data->builtin )
+			return;
+	}
+	p = loadFileIcon(type->fulltypename,&w,&h);
+	if( p != None ) {
+		data->builtin = false;
+		data->icon_bm = p;
+		data->bm_width = w;
+		data->bm_height = h;
+		return;
+	}
+	if( data->icon_bm != None ) {
+		/* a builtin is better than a default,
+		 * so we keep it, and remember that is
+		 * could not be loaded: */
+		data->builtin = false;
+		return;
+	}
+	if( type->fallback != NULL ) {
+		loadtypeicon(type->fallback);
+		data->icon_bm = type->fallback->data->icon_bm;
+		data->bm_width = type->fallback->data->bm_width;
+		data->bm_height = type->fallback->data->bm_height;
+		return;
+	}
+	assert("Missing fallback"==NULL);
+}
+
+static struct mime_filetype *findmimetype(const char *name) {
+	struct mime_filetype *type;
+	
+	type = mime_get_filetype(name);
+	loadtypeicon(type);
+	return type;
 }
 
 /*---------------------------------------------------------------------------*/
 /* initialise the file Windows module */
-void initFileWindows()
+static void initFiles(void)
+{
+	int r;
+
+	r = mime_parse_types(resources.mimetypes_file,loadtypeicon);
+	if( r == 0 )
+		r = mime_parse_types(resources.system_mimetypes_file,loadtypeicon);
+	if( r <= -2 )
+		abortXfm("Out of Memory");
+	r = mime_parse_mailcap(resources.mailcap_file);
+	if( r == 0 )
+		r = mime_parse_mailcap(resources.system_mailcap_file);
+	if( r <= -2 )
+		abortXfm("Out of Memory");
+	if( !exists(resources.dev_file) )
+		readDevices(resources.system_dev_file);
+	else
+		readDevices(resources.dev_file);
+	if( !exists(resources.magic_file) )
+		magic_parse_file(resources.system_magic_file,findmimetype, resources.maxMagicRead);
+	else
+		magic_parse_file(resources.magic_file,findmimetype, resources.maxMagicRead);
+}
+
+void initFileWindows(void)
 {
   XtAppAddActions(app_context, file_actions, XtNumber(file_actions));
 #ifndef ENHANCE_TRANSLATIONS
@@ -1092,34 +1030,24 @@ void initFileWindows()
   label_args[3].value = (XtArgVal) resources.label_font;
   status_args[3].value = (XtArgVal) resources.status_font;
   icon_label_args[3].value = (XtArgVal) resources.icon_font;
-  shell_args[1].value = (XtArgVal) bm[ICON_BM];
-  shell_args[2].value = (XtArgVal) bm[ICONMSK_BM];
+  shell_args[1].value = (XtArgVal) bm_icon;
+  shell_args[2].value = (XtArgVal) bm_iconmsk;
 
-  file_popup_items = createFloatingMenu("file popup", file_popup_menu,
-					XtNumber(file_popup_menu), 4, aw.shell,
-					NULL, &file_popup_widget);
   XtRegisterGrabAction(filePopup, True, ButtonPressMask | ButtonReleaseMask,
 		       GrabModeAsync, GrabModeAsync);
-  dir_popup_items = createFloatingMenu("dir popup", dir_popup_menu,
-					XtNumber(dir_popup_menu), 4, aw.shell,
-					NULL, &dir_popup_widget);
   XtRegisterGrabAction(dirPopup, True, ButtonPressMask | ButtonReleaseMask,
 		       GrabModeAsync, GrabModeAsync);
-  readFileTypes(resources.cfg_file);
-  readDevices(resources.dev_file);
-#ifdef MAGIC_HEADERS
-  magic_parse_file(resources.magic_file);
-#endif
+  initFiles();
 }
 
 /*---------------------------------------------------------------------------*/
 /* Create a file Window at the specified path, in the specified format */
 
-static FileWindowRec *createFileWindow(String path, String title, 
+static FileWindowRec *createFileWindow(const char *path, const char *title,
 				       DisplayType format)
 {
   FileWindowRec *fw;
-  char *shell_name;
+  const char *shell_name;
 
 #ifdef DEBUG_MALLOC
   fprintf(stderr, "entering createFileWindow: %lu\n", malloc_inuse(NULL));
@@ -1184,15 +1112,15 @@ static FileWindowRec *createFileWindow(String path, String title,
 					 XtNumber(button_box_args) );
   
   /* create the menus */
-  fw->file_items = createMenu("file", "File", file_menu, XtNumber(file_menu),
-			      4, fw->button_box, (XtPointer) fw);
-  fw->folder_items = createMenu("folder", "Folder", folder_menu, 
+  fw->file_items = createFmMenu("file", "File", file_menu, XtNumber(file_menu),
+			      4, fw->button_box, fw);
+  fw->folder_items = createFmMenu("folder", "Folder", folder_menu, 
 				XtNumber(folder_menu), 4, fw->button_box,
-				(XtPointer) fw);
-  fw->view_items = createMenu("view", "View", view_menu, XtNumber(view_menu),
-			      16, fw->button_box, (XtPointer) fw);
-  fw->option_items=createMenu("options","Options",text_opts_menu,XtNumber(text_opts_menu),
-			      16, fw->button_box, (XtPointer) fw);
+				fw);
+  fw->view_items = createFmMenu("view", "View", view_menu, XtNumber(view_menu),
+			      16, fw->button_box, fw);
+  fw->option_items=createFmMenu("options","Options",text_opts_menu,XtNumber(text_opts_menu),
+			      16, fw->button_box, fw);
   XtSetMappedWhenManaged(XtParent(XtParent(fw->option_items[0])),False);
   fw->option_in_use=0;
   /* count the number of textFileLists currently
@@ -1222,6 +1150,14 @@ static FileWindowRec *createFileWindow(String path, String title,
   fw->status = XtCreateManagedWidget("status", labelWidgetClass, fw->form,
 				     status_args, XtNumber(status_args) );
   
+  /* create popup menus */
+  fw->file_popup_items = createFloatingMenu("file popup",
+	(const MenuItemRec *)file_popup_menu, XtNumber(file_popup_menu),
+	4, fw->shell, fw, &fw->file_popup_widget);
+  fw->dir_popup_items = createFloatingMenu("dir popup",
+	(const MenuItemRec *)dir_popup_menu, XtNumber(dir_popup_menu),
+	4, fw->shell, fw, &fw->dir_popup_widget);
+
 #ifdef DEBUG_MALLOC
   fprintf(stderr, "exiting createFileWindow: %lu\n", malloc_inuse(NULL));
 #endif
@@ -1349,7 +1285,7 @@ void updateFileDisplay(FileWindowRec *fw)
   x=y=0;
   if (fw->icon_box) {
     if (keep_position)
-      XtVaGetValues(fw->icon_box,XtNx,&x,XtNy,&y,0);
+      XtVaGetValues(fw->icon_box,XtNx,&x,XtNy,&y,(char*)0);
     XtUnrealizeWidget(fw->icon_box);
     XtDestroyWidget(fw->icon_box);
   }
@@ -1504,7 +1440,7 @@ static Boolean files_changed(FileWindowRec *fw)
   if (chdir(fw->directory)) return True;
   for (i=0; i<fw->n_files; i++) {
     statfailed=stat(fl[i]->name,&cur);
-    if (fl[i]->sym_link) {
+    if (fl[i]->info.symlink) {
 	if ( (statfailed && !S_ISLNK(fl[i]->stats.st_mode)) ||
 	     (!statfailed && S_ISLNK(fl[i]->stats.st_mode)) ) {
 	  return True;
@@ -1522,7 +1458,7 @@ static Boolean files_changed(FileWindowRec *fw)
 }
 #endif
 
-void intUpdate()
+void intUpdate(void)
 {
   FileWindowRec *fw;
   struct stat cur;
@@ -1684,7 +1620,7 @@ void updateStatus(FileWindowRec *fw)
 
 void updateTxtOpts(FileWindowRec *fw)
 {
- int i;
+ Cardinal i;
  WidgetList menu_items;
  static Arg args[]={
   { XtNshowInode,	(XtArgVal) 0},
