@@ -1,42 +1,69 @@
-/*---------------------------------------------------------------------------
-  Module FmFwActions
+/*
+ *  This file is part of X-File Manager XFM
+ *  ----------------------------------------------------------------------
+  FmFwActions.c
 
   (c) Simon Marlow 1990-92
   (c) Albert Graef 1994
 
   modified 7-1997 by strauman@sun6hft.ee.tu-berlin.de to add
-  different enhancements (see README-NEW).
+  different enhancements (see README-1.4).
+
+  modified 2004,2005,2006 by Bernhard R. Link (see Changelog)
 
   Action procedures for widgets in a file window
----------------------------------------------------------------------------*/
+ *  ----------------------------------------------------------------------
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <xfmconfig.h>
 
 #include <stdio.h>
 #include <unistd.h>
+#include <assert.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
-#include <X11/Xaw/Toggle.h>
+#include <X11/Xaw3d/Toggle.h>
 #include "FileList.h"
 
+#include "global.h"
+
 #include "Am.h"
 #include "Fm.h"
+#include "execute.h"
 
 #ifdef ENHANCE_SELECTION
 #include "FmSelection.h"
 #endif
 
+#include "mime.h"
+#include "mime_start.h"
+
 /*---------------------------------------------------------------------------
   PUBLIC DATA
 ---------------------------------------------------------------------------*/
 
 MoveInfo move_info;
 Boolean dragging = False;
-Cursor drag_cursor, cache_cursor;
 
 /*---------------------------------------------------------------------------
   STATIC DATA
 ---------------------------------------------------------------------------*/
+static Cursor drag_cursor;
+static Boolean was_dropable;
 
 static char from[MAXPATHLEN], to[MAXPATHLEN];
 static int fromi, toi;
@@ -70,7 +97,7 @@ static int findWidget(Widget w, FileWindowRec **fw_ret, XEvent *ev)
 
 /*---------------------------------------------------------------------------*/
 
-static int moveFiles()
+static int moveFiles(void)
 {
   int i, n_moved = 0;
 
@@ -97,11 +124,12 @@ static int moveFiles()
 	char s[0xff];
 	sprintf(s, "Move: file %s already exists at destination",
 		move_info.fw->files[i]->name);
-	if (!confirm(s, "Overwrite?", ""))
+	if (!confirm(s, "Overwrite?", "")) {
 	  if (aborted)
 	    break;
 	  else
 	    continue;
+	}
       }
       if (rmove(from,to)) {
 	char s[0xff];
@@ -116,7 +144,7 @@ static int moveFiles()
 
 /*---------------------------------------------------------------------------*/
 
-static int copyFiles()
+static int copyFiles(void)
 {
   int i, n_copied = 0;
 
@@ -143,11 +171,12 @@ static int copyFiles()
 	char s[0xff];
 	sprintf(s, "Copy: file %s already exists at destination",
 		move_info.fw->files[i]->name);
-	if (!confirm(s, "Overwrite?", ""))
+	if (!confirm(s, "Overwrite?", "")) {
 	  if (aborted)
 	    break;
 	  else
 	    continue;
+	}
       }
       if (rcopy(from,to)) {
 	char s[0xff];
@@ -262,32 +291,57 @@ static void fileEndCopy(FileWindowRec *fw, int i)
   freeze = False;
 }
 
-/*---------------------------------------------------------------------------*/
+/*-------------------------------------------------------------------------*/
 
-static void fileEndExec(FileWindowRec *fw, int i)
-{
-  int l;
-  char *path;
-  char **argv;
+static const char **fileGetSelected(void) {
+	FileList files = move_info.fw->files;
+	int i, l, j;
+	const char **argv;
 
-  l = strlen(fw->directory);
-  path = (char *)alloca(l+strlen(fw->files[i]->name)+2);
-  strcpy(path, fw->directory);
-  if (path[l-1] != '/')
-    path[l++] = '/';
-  strcpy(path+l, fw->files[i]->name);
+	l = 0;
+	for (j=0; j<move_info.fw->n_files; j++) {
+		if (files[j]->selected) {
+			l++;
+		}
+	}
+	l += 2;
+	argv = (const char**)XtMalloc(l*sizeof(char *));
+	i = 0;
 
-  argv = (char **) XtMalloc(2 * sizeof(char *));
-  argv[0] = XtNewString(path);
-  argv[1] = NULL;
+	for (j=0; j<move_info.fw->n_files; j++) {
+		if (files[j]->selected) {
+			argv[i++] = files[j]->name;
+		}
+	}
+	argv[i++] = NULL;
+	assert(i+1 == l);
 
-  argv = expandArgv(argv);
-  executeApplication(path, move_info.fw->directory, argv);
-  freeArgv(argv);
+	return argv;
 }
 
 /*---------------------------------------------------------------------------*/
 
+void fileCallWithSelected(const char *action) {
+	const char **argv;
+
+	argv = fileGetSelected();
+	argv = makeShellArgs(action, argv);
+
+	execute_external_program(argv[0], move_info.fw->directory, argv,
+			NULL, NULL);
+	XTFREE(argv);
+}
+
+/*---------------------------------------------------------------------------*/
+
+// there was an fileEndExec before here, too.
+// If readding this feature, make it check
+// for the action EXEC and let it use the
+// program itself then....
+//
+// TODO: re-add this one:
+
+/*
 static void fileEndAction(FileWindowRec *fw, int i)
 {
   int l;
@@ -310,13 +364,14 @@ static void fileEndAction(FileWindowRec *fw, int i)
   executeApplication(user.shell, move_info.fw->directory, argv);
   freeArgv(argv);
 }
+*/
 
 /*---------------------------------------------------------------------------*/
 
 /* perform a click-drag, returning the widget that the user released the 
    button on */
 
-static Widget drag(Widget w, int button, FileWindowRec *fw,
+static Widget drag(Widget w, unsigned int button, FileWindowRec *fw,
 		   Boolean *on_root_return, XEvent *event_return)
 {
   Cursor c;
@@ -350,7 +405,8 @@ static Widget drag(Widget w, int button, FileWindowRec *fw,
     move_info.type = MultipleFiles;
     c = curs[FILES_CUR];
   }
-  cache_cursor=drag_cursor = c;
+  drag_cursor = c;
+  was_dropable = true;
 
   XGrabPointer(XtDisplay(w), XtWindow(w), True,
 	       EnterWindowMask | LeaveWindowMask | ButtonReleaseMask, 
@@ -376,7 +432,6 @@ static Widget drag(Widget w, int button, FileWindowRec *fw,
   }
 
   XUngrabPointer(XtDisplay(aw.shell),CurrentTime);
-  cache_cursor=(Cursor)-1;
 
   XtDispatchEvent(event_return);
   freeze = dragging = False;
@@ -400,6 +455,16 @@ static Widget drag(Widget w, int button, FileWindowRec *fw,
   PUBLIC FUNCTIONS
 ---------------------------------------------------------------------------*/
 
+void drag_set_dropable(Display *d, bool dropable) {
+
+	if (was_dropable != dropable) {
+		XChangeActivePointerGrab(d,
+		       EnterWindowMask | LeaveWindowMask | ButtonReleaseMask, 
+		       dropable?drag_cursor:curs[NOENTRY_CUR], CurrentTime);
+		was_dropable = dropable;
+	}
+}
+
 void filePopup(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
   int i;
@@ -417,15 +482,14 @@ void filePopup(Widget w, XEvent *event, String *params, Cardinal *num_params)
     return;
   }
 
-  popup_fw = fw;
   fileSelect(w, event, params, num_params);
 
   if (S_ISLNK(fw->files[i]->stats.st_mode)) {
-    grayOut(file_popup_items[0]);
-    grayOut(file_popup_items[1]);
+    grayOut(fw->file_popup_items[0]);
+    grayOut(fw->file_popup_items[1]);
   } else {
-    fillIn(file_popup_items[0]);
-    fillIn(file_popup_items[1]);
+    fillIn(fw->file_popup_items[0]);
+    fillIn(fw->file_popup_items[1]);
   }
 
   dpy = XtDisplay(aw.shell);
@@ -433,10 +497,10 @@ void filePopup(Widget w, XEvent *event, String *params, Cardinal *num_params)
   XQueryPointer(dpy, DefaultRootWindow(dpy), &root, &child, &x, &y, 
 		&x_win, &y_win, &mask);
   
-  XtVaSetValues(file_popup_widget, XtNx, (XtArgVal) x, XtNy, (XtArgVal) y,
+  XtVaSetValues(fw->file_popup_widget, XtNx, (XtArgVal) x, XtNy, (XtArgVal) y,
 		NULL);
   
-  XtPopupSpringLoaded(file_popup_widget);
+  XtPopupSpringLoaded(fw->file_popup_widget);
 }  
 
 /*---------------------------------------------------------------------------*/
@@ -458,20 +522,19 @@ void dirPopup(Widget w, XEvent *event, String *params, Cardinal *num_params)
     return;
   }
 
-  popup_fw = fw;
   fileSelect(w, event, params, num_params);
 
   if (!strcmp(fw->files[i]->name, ".") ||
       !strcmp(fw->files[i]->name, "..")) {
-    grayOut(dir_popup_items[2]);
-    grayOut(dir_popup_items[4]);
-    grayOut(dir_popup_items[5]);
-    grayOut(dir_popup_items[8]);
+    grayOut(fw->dir_popup_items[2]);
+    grayOut(fw->dir_popup_items[4]);
+    grayOut(fw->dir_popup_items[5]);
+    grayOut(fw->dir_popup_items[8]);
   } else {
-    fillIn(dir_popup_items[2]);
-    fillIn(dir_popup_items[4]);
-    fillIn(dir_popup_items[5]);
-    fillIn(dir_popup_items[8]);
+    fillIn(fw->dir_popup_items[2]);
+    fillIn(fw->dir_popup_items[4]);
+    fillIn(fw->dir_popup_items[5]);
+    fillIn(fw->dir_popup_items[8]);
   }
 
   dpy = XtDisplay(aw.shell);
@@ -479,15 +542,16 @@ void dirPopup(Widget w, XEvent *event, String *params, Cardinal *num_params)
   XQueryPointer(dpy, DefaultRootWindow(dpy), &root, &child, &x, &y, 
 		&x_win, &y_win, &mask);
   
-  XtVaSetValues(dir_popup_widget, XtNx, (XtArgVal) x, XtNy, (XtArgVal) y,
+  XtVaSetValues(fw->dir_popup_widget, XtNx, (XtArgVal) x, XtNy, (XtArgVal) y,
 		NULL);
   
-  XtPopupSpringLoaded(dir_popup_widget);
+  XtPopupSpringLoaded(fw->dir_popup_widget);
 }  
 
 /*---------------------------------------------------------------------------*/
 
-void fileToggle(Widget w, XEvent *event, String *params, Cardinal *num_params)
+void fileToggle(Widget w, XEvent *event,
+		UNUSED(String *params), UNUSED(Cardinal *num_params))
 {
   int i;
   
@@ -528,7 +592,8 @@ void fileToggle(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
 /*---------------------------------------------------------------------------*/
 
-void fileSelect(Widget w, XEvent *event, String *params, Cardinal *num_params)
+void fileSelect(Widget w, XEvent *event,
+		UNUSED(String *params), UNUSED(Cardinal *num_params))
 {
   int i, j;
   int onewid=XtIsSubclass(w,fileListWidgetClass);
@@ -571,12 +636,11 @@ void fileSelect(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
 /*---------------------------------------------------------------------------*/
 
-void fileHighlight(Widget w, XEvent *event, String *params, 
-		   Cardinal *num_params)
+void fileHighlight(Widget w, XEvent *event,
+		   UNUSED(String *params), UNUSED(Cardinal *num_params))
 {
   int i;
   FileWindowRec *fw;
-  Cursor	c;
 
   if (!dragging)
     return;
@@ -591,15 +655,10 @@ void fileHighlight(Widget w, XEvent *event, String *params,
 
   if ((i == -1 && !permission(&fw->stats,W_OK)) || 
       (i != -1 && S_ISDIR(fw->files[i]->stats.st_mode) && 
-      !permission(&fw->files[i]->stats, W_OK))) c=curs[NOENTRY_CUR];
-  else					        c=drag_cursor;
-
-  if (c!=cache_cursor) {
-    XChangeActivePointerGrab(XtDisplay(w),
-		       EnterWindowMask | LeaveWindowMask | ButtonReleaseMask, 
-		       c, CurrentTime);
-    cache_cursor=c;
-  }
+      !permission(&fw->files[i]->stats, W_OK)))
+	  drag_set_dropable(XtDisplay(w),False);
+  else
+	  drag_set_dropable(XtDisplay(w),True);
 
   if ( i != -1 && (fw != move_info.fw || !fw->files[i]->selected))
     XtCallActionProc(w, "highlight", event, NULL, 0);
@@ -607,40 +666,30 @@ void fileHighlight(Widget w, XEvent *event, String *params,
 
 /*---------------------------------------------------------------------------*/
 
-void trackCursor(Widget w, XEvent *event, String *params,
-		 Cardinal *num_params)
+void trackCursor(Widget w, XEvent *event, UNUSED(String *params),
+		 UNUSED(Cardinal *num_params))
 {
-  FileWindowRec *fw;
-  Cursor	c;
+	FileWindowRec *fw;
 
-  if (!dragging) return;
+	if (!dragging) return;
 
-  c=drag_cursor;
-
-  if (event->type!=LeaveNotify) {
-  fw=file_windows;
-  while(fw) {
-    if (w==fw->viewport || w==fw->icon_box ||
-        w==fw->label || XtParent(w)==fw->viewport) {
-          if (!permission(&fw->stats,W_OK)) c=curs[NOENTRY_CUR];
-	  break;
-    }
-    fw=fw->next;
-  }
-  }
-  if (c!=cache_cursor) {
-    XChangeActivePointerGrab(XtDisplay(w),
-		       EnterWindowMask | LeaveWindowMask | ButtonReleaseMask, 
-		       c, CurrentTime);
-    cache_cursor=c;
-  }
-  
+	for (fw = file_windows ; fw != NULL ; fw = fw->next) {
+		if (w != fw->viewport)
+			continue;
+		if (event->type==LeaveNotify)
+			drag_set_dropable(XtDisplay(w), False);
+		else if (!permission(&fw->stats,W_OK))
+			drag_set_dropable(XtDisplay(w), False);
+		else
+			drag_set_dropable(XtDisplay(w), True);
+		return;
+	}
 }
 
 /*---------------------------------------------------------------------------*/
 
-void fileMaybeHighlight(Widget w, XEvent *event, String *params, 
-			Cardinal *num_params)
+void fileMaybeHighlight(Widget w, XEvent *event,
+		        UNUSED(String *params), UNUSED(Cardinal *num_params))
 {
   int i;
   FileWindowRec *fw;
@@ -650,14 +699,12 @@ void fileMaybeHighlight(Widget w, XEvent *event, String *params,
 
   i = findWidget(w,&fw, event);
   if (!fw) {
-/*
-    error("Internal error:","Widget not found in fileMaybeHighlight");
-*/
     return;
   }
 
+  /*
   if (i != -1 && (fw != move_info.fw || !fw->files[i]->selected) &&
-      fw->files[i]->type && *fw->files[i]->type->drop_action) {
+      fw->files[i]->mime_type && *fw->files[i]->mime_type->action->drop_action) {
     if (cache_cursor!=drag_cursor) {
       XChangeActivePointerGrab(XtDisplay(w),
 		       EnterWindowMask | LeaveWindowMask | ButtonReleaseMask, 
@@ -666,14 +713,15 @@ void fileMaybeHighlight(Widget w, XEvent *event, String *params,
     }
     XtCallActionProc(w, "highlight", event, NULL, 0);
   }
+  */
 }
 
 /*---------------------------------------------------------------------------*/
 
-void resetCursor(Widget w, XEvent *event, String *params,Cardinal *num_params)
+void resetCursor(Widget w, XEvent *event,
+		 UNUSED(String *params), UNUSED(Cardinal *num_params))
 {
   int i;
-  Cursor c;
   FileWindowRec *fw;
   i = findWidget(w,&fw, event);
   if (!fw) {
@@ -686,20 +734,24 @@ void resetCursor(Widget w, XEvent *event, String *params,Cardinal *num_params)
   if (!dragging)
     return;
 
-  c=((!permission(&fw->stats,W_OK))?curs[NOENTRY_CUR]:drag_cursor);
+  if (event->type != LeaveNotify)
+	  abortXfm("resetCursor called with event not of type LeaveNotify");
 
-  if (c!=cache_cursor) {
-    XChangeActivePointerGrab(XtDisplay(w),
-		      EnterWindowMask | LeaveWindowMask | ButtonReleaseMask, 
-		      c,CurrentTime);
-    cache_cursor=c;
-  }
+  if (!event->xcrossing.send_event )
+	  /* real leave event, so the cursor is outside the window */
+	  drag_set_dropable(XtDisplay(w), False);
+  else if (!permission(&fw->stats,W_OK))
+	  /* the current directory is not writeable */
+	  drag_set_dropable(XtDisplay(w), False);
+  else
+	  drag_set_dropable(XtDisplay(w), True);
 }
 
 
 /*---------------------------------------------------------------------------*/
 
-void fileRefresh(Widget w, XEvent *event, String *params, Cardinal *num_params)
+void fileRefresh(Widget w, XEvent *event,
+		 UNUSED(String *params), UNUSED(Cardinal *num_params))
 {
   FileWindowRec *fw;
   
@@ -714,7 +766,8 @@ void fileRefresh(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
 /*---------------------------------------------------------------------------*/
 
-void fileOpenDir(Widget w, XEvent *event, String *params,Cardinal *num_params)
+void fileOpenDir(Widget w, XEvent *event,
+		 UNUSED(String *params), UNUSED(Cardinal *num_params))
 {
   FileWindowRec *fw;
   int i;
@@ -793,18 +846,13 @@ void fileBeginDrag(Widget w, XEvent *event, String *params,
       appEndDragInBox();
     else if ((i = findAppWidget(w)) != -1)
       appEndDrag(i);
-/*
-    else
-      error("Internal error:","widget not found in fileBeginDrag");
-*/
   }
 
   /* Otherwise, if we didn't go anywhere, don't do anything */
   else if (fw == move_info.fw &&
       (i == -1 || fw->files[i]->selected ||
-       (!S_ISDIR(fw->files[i]->stats.st_mode) &&
-	!(fw->files[i]->stats.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) &&
-	!(fw->files[i]->type && *fw->files[i]->type->drop_action))))
+       (!S_ISDIR(fw->files[i]->stats.st_mode) 
+      )))
     ;
 
   /* Otherwise, if dragged onto a window or a directory, do the move */
@@ -814,13 +862,11 @@ void fileBeginDrag(Widget w, XEvent *event, String *params,
     else
       fileEndCopy(fw,i);
 
-  /* Otherwise, if dragged onto an executable, run the program */
-  else if (fw->files[i]->stats.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
-    fileEndExec(fw,i);
-
   /* Otherwise, if the file has a drop action, invoke the action */
+/*  
   else if (fw->files[i]->type && *fw->files[i]->type->drop_action)
     fileEndAction(fw,i);
+*/
 
   /* Otherwise, it must be a normal file so just do the move */
   else if (move)
@@ -830,44 +876,11 @@ void fileBeginDrag(Widget w, XEvent *event, String *params,
 }
   
 /*---------------------------------------------------------------------------*/
-
-void fileExecFile(Widget w, XEvent *event, String *params, 
-		  Cardinal *num_params)
-{
-  int i, l;
-  char *path;
-  char **argv;
-  FileWindowRec *fw;
-
-  i = findWidget(w, &fw, event);
-  if (i == -1 || !fw) {
-/*
-    error("Internal error:","widget not found in fileExecFile");
-*/
-    return;
-  }
-
-  l = strlen(fw->directory);
-  path = (char *)alloca(l+strlen(fw->files[i]->name)+2);
-  strcpy(path, fw->directory);
-  if (path[l-1] != '/')
-    path[l++] = '/';
-  strcpy(path+l, fw->files[i]->name);
-  argv = (char **) XtMalloc(2 * sizeof(char *));
-  argv[0] = XtNewString(fw->files[i]->name);
-  argv[1] = NULL;
-  executeApplication(path, fw->directory, argv);
-
-  freeArgv(argv);
-}
-
-/*---------------------------------------------------------------------------*/
-void fileExecAction(Widget w, XEvent *event, String *params, 
-		    Cardinal *num_params)
+void fileExecAction(Widget w, XEvent *event,
+		    UNUSED(String *params), UNUSED(Cardinal *num_params))
 {
   int i;
   FileWindowRec *fw;
-  char **argv;
 
   i = findWidget(w, &fw, event);
   if (i == -1 || !fw) {
@@ -876,124 +889,51 @@ void fileExecAction(Widget w, XEvent *event, String *params,
 */
     return;
   }
-
-  if (fw->files[i]->type) {
-    if (*fw->files[i]->type->push_action)
-      if (!strcmp(fw->files[i]->type->push_action, "EDIT"))
-	doEdit(fw->directory, fw->files[i]->name);
-      else if (!strcmp(fw->files[i]->type->push_action, "VIEW"))
-	doView(fw->directory, fw->files[i]->name);
-      else if (!strcmp(fw->files[i]->type->push_action, "LOAD")) {
-	int j, l;
-	if (!resources.appmgr) return;
-	pushApplicationsFile();
-	l = strlen(fw->directory);
-	strcpy(resources.app_file, fw->directory);
-	if (resources.app_file[l-1] != '/')
-	  resources.app_file[l++] = '/';
-	strcpy(resources.app_file+l, fw->files[i]->name);
-	for(j=0; j<aw.n_apps; j++)
-	  freeApplicationResources(&aw.apps[j]);
-	XTFREE(aw.apps);
-	readApplicationData(resources.app_file);
-	updateApplicationDisplay();
-      } else {
-        int k = 0;
-	char *action = varPopup(fw->files[i]->type->icon_bm,
-				fw->files[i]->type->push_action);
-
-	if (!action) return;
-
-	argv = (char **) XtMalloc( (user.arg0flag ? 6 : 5) * sizeof(char *));
-	argv[k++] = user.shell;
-	argv[k++] = "-c";
-	argv[k++] = action;
-        if (user.arg0flag)
-          argv[k++] = user.shell;
-	argv[k++] = fw->files[i]->name;
-	argv[k] = NULL;
-
-	executeApplication(user.shell, fw->directory, argv);
-
-	XTFREE(argv);
-      }
+  if (fw->files[i]->mime_type) {
+	  if (!mime_start(fw->files[i]->mime_type,
+			  fw->directory,
+			  fw->files[i]->name,
+			  MIME_START_ANY)) {
+		  /* TODO: show some selection dialog here instead */
+		  error("No action found for type:",fw->files[i]->mime_type->fulltypename);
+	  }
   } else
-    doEdit(fw->directory, fw->files[i]->name);
+    error("Internal error:","missing file type");
 }
 
 /*---------------------------------------------------------------------------*/
-void doEdit(char *directory, char *fname)
+void doEdit(const char *directory, const char *fname)
 {
-  char path[MAXPATHLEN];
-  char **argv;
+  if (resources.editor && resources.editor[0] && resources.editor[0][0]) {
+	  const char **args;
+	  args = appendArgs(resources.editor, 1, fname, NULL);
 
-  if (resources.default_editor) {
-
-    strcpy(path, resources.default_editor);
-    strcat(path, " ");
-    strcat(path, fname);
-
-    argv = (char **) XtMalloc(4 * sizeof(char *));
-    argv[0] = user.shell;
-    argv[1] = "-c";
-    argv[2] = path;
-    argv[3] = NULL;
-
-    executeApplication(user.shell, directory, argv);
-  
-    XTFREE(argv);
+	  execute_external_program(args[0],directory, args, NULL, NULL);
+	  XTFREE(args);
   }
-
   else
     error("No default editor", "");
 }
 /*---------------------------------------------------------------------------*/
-void doView(char *directory, char *fname)
+void doView(const char *directory, const char *fname)
 {
-  char path[MAXPATHLEN];
-  char **argv;
+  if (resources.viewer && resources.viewer[0] && resources.viewer[0][0]) {
+	  const char **args;
+	  args = appendArgs(resources.viewer, 1, fname, NULL);
 
-  if (resources.default_viewer) {
-
-    strcpy(path, resources.default_viewer);
-    strcat(path, " ");
-    strcat(path, fname);
-
-    argv = (char **) XtMalloc(4 * sizeof(char *));
-    argv[0] = user.shell;
-    argv[1] = "-c";
-    argv[2] = path;
-    argv[3] = NULL;
-
-    executeApplication(user.shell, directory, argv);
-  
-    XTFREE(argv);
+	  execute_external_program(args[0],directory, args, NULL, NULL);
+	  XTFREE(args);
   }
-
   else
     error("No default viewer", "");
 }
 /*---------------------------------------------------------------------------*/
-void doXterm(char *directory)
+void doXterm(const char *directory)
 {
-  char path[MAXPATHLEN];
-  char **argv;
-
-  if (resources.default_xterm) {
-
-    strcpy(path, resources.default_xterm);
-
-    argv = (char **) XtMalloc(4 * sizeof(char *));
-    argv[0] = user.shell;
-    argv[1] = "-c";
-    argv[2] = path;
-    argv[3] = NULL;
-
-    executeApplication(user.shell, directory, argv);
-  
-    XTFREE(argv);
+  if (resources.xtermalone && resources.xtermalone[0] && resources.xtermalone[0][0]) {
+	  execute_external_program(resources.xtermalone[0], directory,
+			  resources.xtermalone, NULL, NULL);
   }
-
   else
     error("No default xterm", "");
 }
