@@ -1,15 +1,34 @@
-/*---------------------------------------------------------------------------
- Module FmMain
+/*
+ *  This file is part of X-File Manager XFM
+ *  ----------------------------------------------------------------------
+ FmMain.c
 
  (c) S.Marlow 1990-92
  (c) A.Graef 1994
  (c) R.Vogelgesang 1994 (`Xfm.BourneShells' stuff)
 
   modified 7-1997 by strauman@sun6hft.ee.tu-berlin.de to add
-  different enhancements (see README-NEW).
+  different enhancements (see README-1.4).
+
+  modified 2003,2004,2005,2006 by Bernhard R. Link (see Changelog)
 
  main module for file manager    
----------------------------------------------------------------------------*/
+ *  ----------------------------------------------------------------------
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+#include <xfmconfig.h>
 
 #include "FmVersion.h"
 
@@ -37,11 +56,14 @@
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
-#include <X11/Xaw/Cardinals.h>
+#include <X11/Xaw3d/Cardinals.h>
 #include <X11/Shell.h>
 
+#include "global.h"
 #include "Am.h"
 #include "Fm.h"
+#include "mime.h"
+#include "mime_start.h"
 
 #ifdef ENHANCE_SELECTION
 #include "FmSelection.h"
@@ -50,9 +72,21 @@
 #ifdef ENHANCE_LOG
 #include "FmLog.h"
 #endif
+#include "execute.h"
 
 #define XtRDisplayType "DisplayType"
 #define XtRSortType "SortType"
+#define XtRExpandedString "ExpandedString"
+
+#ifdef TESTRESOURCES
+#define XFM_APP_CLASS "TestXfm"
+#else
+#define XFM_APP_CLASS "Xfm"
+#endif
+
+#define XFM_BITMAP_PATH PKGDATADIR "/bitmaps:/usr/include/X11/bitmaps"
+#define XFM_PIXMAP_PATH PKGDATADIR "/pixmaps:/usr/include/X11/pixmaps:/usr/share/pixmaps"
+#define XFM_ICON_PATH PKGDATADIR "/icons"
 
 /*---------------------------------------------------------------------------
   Public variables
@@ -122,24 +156,46 @@ static XtResource resource_list[] = {
       XtOffsetOf(Resources, tree_icon_width), XtRImmediate, (XtPointer) 48 },
   { "treeIconHeight", "Height", XtRInt, sizeof(int),
       XtOffsetOf(Resources, tree_icon_height), XtRImmediate, (XtPointer) 32 },
-  { "bitmapPath", "Path", XtRString, sizeof(String),
-      XtOffsetOf(Resources, bitmap_path), XtRString, NULL },
-  { "pixmapPath", "Path", XtRString, sizeof(String),
-      XtOffsetOf(Resources, pixmap_path), XtRString, NULL },
-  { "applicationDataFile", "ConfigFile",  XtRString, sizeof(String),
-      XtOffsetOf(Resources, app_file_r), XtRString, "~/.xfm/Apps" },
-  { "applicationDataDir", "ConfigDir",  XtRString, sizeof(String),
-      XtOffsetOf(Resources, app_dir_r), XtRString, "~/.xfm" },
-  { "applicationDataClip", "File",  XtRString, sizeof(String),
-      XtOffsetOf(Resources, app_clip_r), XtRString, "~/.xfm/.XfmClip" },
-  { "configFile", "ConfigFile",  XtRString, sizeof(String),
-      XtOffsetOf(Resources, cfg_file_r), XtRString, "~/.xfm/xfmrc" },
-  { "devFile", "ConfigFile",  XtRString, sizeof(String),
-      XtOffsetOf(Resources, dev_file_r), XtRString, "~/.xfm/xfmdev" },
-#ifdef MAGIC_HEADERS
-  { "magicFile", "ConfigFile",  XtRString, sizeof(String),
-      XtOffsetOf(Resources, magic_file_r), XtRString, "~/.xfm/magic" },
+  { "bitmapPath", "Path", XtRExpandedString, sizeof(String),
+      XtOffsetOf(Resources, bitmap_path), XtRString, XFM_BITMAP_PATH },
+  { "pixmapPath", "Path", XtRExpandedString, sizeof(String),
+      XtOffsetOf(Resources, pixmap_path), XtRString, XFM_PIXMAP_PATH },
+  { "iconPath", "Path", XtRExpandedString, sizeof(String),
+      XtOffsetOf(Resources, icon_path), XtRString, XFM_ICON_PATH },
+  { "symlinkDir", "ConfigDir",  XtRExpandedString, sizeof(String),
+      XtOffsetOf(Resources, symlinkdir), XtRString, "/tmp/xfm-${USER}-${PID}" },
+  { "alwaysSymlink", "Safety", XtRBoolean, sizeof(Boolean),
+      XtOffsetOf(Resources, alwaysSymlink), XtRImmediate, (XtPointer)False },
+  { "applicationDataFile", "ConfigFile",  XtRExpandedString, sizeof(String),
+      XtOffsetOf(Resources, main_app_file), XtRString, "~/.xfm/Apps.xfm" },
+  { "applicationDataDir", "ConfigDir",  XtRExpandedString, sizeof(String),
+      XtOffsetOf(Resources, app_dir), XtRString, "~/.xfm" },
+  { "applicationDataClip", "File",  XtRExpandedString, sizeof(String),
+      XtOffsetOf(Resources, app_clip), XtRString, "~/.xfm/.XfmClip" },
+  { "devFile", "ConfigFile",  XtRExpandedString, sizeof(String),
+      XtOffsetOf(Resources, dev_file), XtRString, "~/.xfm/xfm_dev" },
+  { "mailcapFile", "ConfigFile", XtRExpandedString, sizeof(String),
+      XtOffsetOf(Resources, mailcap_file), XtRString, "~/.xfm/xfm_mailcap" },
+  { "mimeTypesFile", "ConfigFile", XtRExpandedString, sizeof(String),
+      XtOffsetOf(Resources, mimetypes_file), XtRString, "~/.xfm/xfm_mime.types" },
+#ifdef SYSTEMWIDE_DEFAULTS
+  { "systemwideApplicationDataFile", "ConfigFile",  XtRExpandedString, sizeof(String),
+      XtOffsetOf(Resources, system_main_app_file), XtRString, XFM_APPDIR "/Apps.xfm" },
+  { "systemwideApplicationDataDir", "ConfigFile",  XtRExpandedString, sizeof(String),
+      XtOffsetOf(Resources, system_app_dir), XtRString, XFM_APPDIR },
+  { "systemwideDevFile", "ConfigFile",  XtRExpandedString, sizeof(String),
+      XtOffsetOf(Resources, system_dev_file), XtRString, XFM_CONFDIR "/xfm_dev" },
+  { "systemwideMailcapFile", "ConfigFile", XtRExpandedString, sizeof(String),
+      XtOffsetOf(Resources, system_mailcap_file), XtRString, XFM_CONFDIR "/xfm_mailcap" },
+  { "systemwideMagicFile", "ConfigFile",  XtRExpandedString, sizeof(String),
+      XtOffsetOf(Resources, system_magic_file), XtRString, XFM_CONFDIR "/xfm_magic" },
+  { "systemwideMimeTypesFile", "ConfigFile", XtRString, sizeof(String),
+      XtOffsetOf(Resources, system_mimetypes_file), XtRString, XFM_CONFDIR "/xfm_mime.types" },
 #endif
+  { "magicFile", "ConfigFile",  XtRExpandedString, sizeof(String),
+      XtOffsetOf(Resources, magic_file), XtRString, "~/.xfm/xfm_magic" },
+  { "maxMagicRead", XtCValue, XtRCardinal, sizeof(Cardinal),
+      XtOffsetOf(Resources, maxMagicRead), XtRImmediate, (XtPointer) 1024 },
   { "confirmDeletes", "Confirm", XtRBoolean, sizeof(Boolean),
       XtOffsetOf(Resources, confirm_deletes), XtRImmediate, (XtPointer) True },
   { "confirmDeleteFolder", "Confirm", XtRBoolean, sizeof(Boolean),
@@ -156,6 +212,8 @@ static XtResource resource_list[] = {
       XtOffsetOf(Resources, confirm_quit), XtRImmediate, (XtPointer) True },
   { "echoActions", "Echo", XtRBoolean, sizeof(Boolean),
       XtOffsetOf(Resources, echo_actions), XtRImmediate, (XtPointer) False },
+  { "echoMimeSearch", "Echo", XtRBoolean, sizeof(Boolean),
+      XtOffsetOf(Resources, echo_mime_search), XtRImmediate, (XtPointer) False },
   { "showOwner", "ShowOwner", XtRBoolean, sizeof(Boolean),
       XtOffsetOf(Resources, show_owner), XtRImmediate, (XtPointer) True },
   { "showDate", "ShowDate", XtRBoolean, sizeof(Boolean),
@@ -177,7 +235,7 @@ static XtResource resource_list[] = {
       XtRImmediate, (XtPointer) Icons },
   { "initialDisplayType", "InitialDisplayType", XtRDisplayType, 
       sizeof(DisplayType), XtOffsetOf(Resources, initial_display_type),
-      XtRImmediate, (XtPointer) Tree },
+      XtRImmediate, (XtPointer) Icons },
   { "defaultSortType", "DefaultSortType", XtRSortType, 
       sizeof(SortType), XtOffsetOf(Resources, default_sort_type),
       XtRImmediate, (XtPointer) SortByName },
@@ -191,18 +249,16 @@ static XtResource resource_list[] = {
   { "hardUpdateTicks", "HardUpdateTicks", XtRInt, sizeof(int),
       XtOffsetOf(Resources, hard_update_ticks), XtRImmediate, (XtPointer)0 },
 #endif
-  { "defaultEditor", "DefaultEditor", XtRString, sizeof(String),
-      XtOffsetOf(Resources, default_editor), XtRString, NULL },
-  { "defaultViewer", "DefaultViewer", XtRString, sizeof(String),
-      XtOffsetOf(Resources, default_viewer), XtRString, NULL },
-  { "defaultXterm", "DefaultXterm", XtRString, sizeof(String),
-      XtOffsetOf(Resources, default_xterm), XtRString, NULL },
-  { "BourneShells", "ShellList", XtRString, sizeof(String),
-      XtOffsetOf(Resources, sh_list), XtRString, NULL },
-#ifdef ENHANCE_USERINFO
-  { "shellCommand", "ShellCommand", XtRString, sizeof(String),
-      XtOffsetOf(Resources, sh_cmd), XtRString, NULL },
-#endif
+  { "editor", "Editor", XtRCommandArgArray, sizeof(String),
+      XtOffsetOf(Resources, editor), XtRString, NULL },
+  { "viewer", "Viewer", XtRCommandArgArray, sizeof(String),
+      XtOffsetOf(Resources, viewer), XtRString, NULL },
+  { "xTerminal", "XTerminal", XtRCommandArgArray, sizeof(String*),
+      XtOffsetOf(Resources, xterminal), XtRString, "x-terminal-emulator -e" },
+  { "xTerminalAlone", "XTerminalAlone", XtRCommandArgArray, sizeof(String*),
+      XtOffsetOf(Resources, xtermalone), XtRString, "x-terminal-emulator" },
+  { "shell", "Shell", XtRCommandArgArray, sizeof(String),
+      XtOffsetOf(Resources, shell), XtRString, NULL },
 #ifdef ENHANCE_HISTORY
   { "historyMaxN", "HistoryMaxN", XtRInt, sizeof(int),
       XtOffsetOf(Resources, history_max_n), XtRImmediate, (XtPointer)30 },
@@ -211,12 +267,10 @@ static XtResource resource_list[] = {
   { "appDefsVersion","AppDefsVersion",XtRString,sizeof(String),
       XtOffsetOf(Resources, app_defs_version), XtRString, (XtPointer)"NOT SET"},
 #endif
-#ifdef XPM
 #ifdef ENHANCE_CMAP
   { "colorCloseness","ColorCloseness",XtRInt,sizeof(int),
       XtOffsetOf(Resources, color_closeness), XtRImmediate, (XtPointer)40000},
 #endif
-#endif
 #ifdef ENHANCE_SELECTION
   { "highlightColor","HighlightColor",XtRPixel,sizeof(Pixel),
       XtOffsetOf(Resources, highlight_pixel), XtRString, (XtPointer)XtDefaultForeground},
@@ -230,34 +284,7 @@ static XtResource resource_list[] = {
 ---------------------------------------------------------------------------*/
 
 static String fallback_resources[] = {
-#ifdef ENHANCE_TRANSLATIONS
-  "Xfm.appDefsVersion: \"\"",
-#endif
-  "Xfm*Command.cursor : hand2",
-  "Xfm*MenuButton.cursor : hand2",
-  "Xfm*popup form*bitmap.borderWidth : 0",
-  "Xfm*popup form*label.borderWidth : 0",
-  "Xfm*button box.orientation : horizontal",
-  "Xfm*button box.borderWidth: 0",
-  "Xfm*file window*viewport.borderWidth: 0",
-  "Xfm*viewport.icon box*Label.borderWidth : 0",
-  "Xfm*viewport.icon box.Command.borderWidth : 0",
-  "Xfm*viewport.icon box.Form.borderWidth : 0",
-  "Xfm*viewport.icon box*Toggle.borderWidth : 1",
-  "Xfm*chmod*Label.borderWidth : 0",
-  "Xfm*info*Label.borderWidth : 0",
-  "Xfm*error*Label.borderWidth : 0",
-  "Xfm*confirm*Label.borderWidth : 0",
-#ifndef ENHANCE_TXT_FIELD
-  "Xfm*Text*translations : #override \\n\
-    <Key>Return: no-op() \\n\
-    <Key>Linefeed : no-op() \\n\
-    Ctrl<Key>J : no-op() \\n",
-  "Xfm*Text*baseTranslations : #override \\n\
-    <Key>Return: no-op() \\n\
-    <Key>Linefeed : no-op() \\n\
-    Ctrl<Key>J : no-op() \\n",
-#endif
+#include "../defaults/fallback.res"
 NULL,
 };
 
@@ -269,220 +296,111 @@ static Arg shell_args[] = {
   { XtNtitle, (XtArgVal) "Applications" }
 };
 
-/*-----------------------------------------------------------------------------
-  Signal handler - clears up Zombie processes
-  I'll probably extend this in the future to do something useful.
------------------------------------------------------------------------------*/
-static void sigcldHandler(int i)
-{
-  waitpid(-1,NULL,WNOHANG);
-}
-
-static struct sigaction sigcld, sigterm;
+static struct sigaction sigterm;
 
 /*---------------------------------------------------------------------------
   Resource converter functions
 ---------------------------------------------------------------------------*/
 
-static void CvtStringToDisplayType(XrmValue *args, Cardinal *n_args,
-				   XrmValue *fromVal, XrmValue *toVal)
+static Boolean CvtStringToDisplayType(UNUSED(Display *display),
+		UNUSED(XrmValue *args), UNUSED(Cardinal *n_args),
+		XrmValue *fromVal, XrmValue *toVal,
+		UNUSED(XtPointer *priv_data))
 {
   static DisplayType d;
 
-  if (!strcmp(fromVal->addr, "Tree"))
-    d = Tree;
-  else if (!strcmp(fromVal->addr, "Icons"))
-    d = Icons;
-  else if (!strcmp(fromVal->addr, "Text"))
-    d = Text;
+  if( strcmp(fromVal->addr, "Tree") == 0)
+	  d = Tree;
+  else if( strcmp(fromVal->addr, "Icons") == 0)
+	  d = Icons;
+  else if( strcmp(fromVal->addr, "Text") == 0)
+	  d = Text;
   else {
-    XtStringConversionWarning(fromVal->addr, XtRDisplayType);
-    return;
+	  XtStringConversionWarning(fromVal->addr, XtRDisplayType);
+	  return False;
+  }
+
+  if( sizeof(DisplayType) > toVal->size ) {
+	  toVal->addr = (caddr_t) &d;
+  } else {
+	  *(DisplayType*)toVal->addr = d;
   }
-  
-  toVal->addr = (caddr_t) &d;
   toVal->size = sizeof(DisplayType);
+  return True;
 }
 
 /*---------------------------------------------------------------------------*/
  
-static void CvtStringToSortType(XrmValue *args, Cardinal *n_args,
-				XrmValue *fromVal, XrmValue *toVal)
+static Boolean CvtStringToSortType(UNUSED(Display *display),
+		UNUSED(XrmValue *args), UNUSED(Cardinal *n_args),
+		XrmValue *fromVal, XrmValue *toVal,
+		UNUSED(XtPointer *priv_data))
 {
   static SortType d;
 
-  if (!strcmp(fromVal->addr, "SortByName"))
+  if (strcmp(fromVal->addr, "SortByName") == 0)
     d = SortByName;
-  else if (!strcmp(fromVal->addr, "SortBySize"))
+  else if (strcmp(fromVal->addr, "SortBySize") == 0)
     d = SortBySize;
-  else if (!strcmp(fromVal->addr, "SortByDate"))
+  else if (strcmp(fromVal->addr, "SortByDate") == 0)
     d = SortByMTime;
   else {
     XtStringConversionWarning(fromVal->addr, XtRSortType);
-    return;
+    return False;
   }
   
-  toVal->addr = (caddr_t) &d;
-  toVal->size = sizeof(SortType);
-}
-
-/*---------------------------------------------------------------------------
-  `Xfm.BourneShells' related functions  
----------------------------------------------------------------------------*/  
-
-int shell_test(UserInfo *ui)
-{
-  int pipe_fd[2];
-  int p;
-  char val[3];
-
-  if (pipe(pipe_fd) < 0) {
-    perror("Can't create pipe");
-    exit(1);
-  }
-
-  p = fork();
-  if (p < 0) {
-    perror("Can't fork");
-    exit(1);
-  }
-
-  if (!p) {       /* child; exec the shell w/ test args */
-    dup2(pipe_fd[1], fileno(stdout));
-    if (close(pipe_fd[0]) == -1) {
-      perror("(child) Can't close pipe");
-      exit(1);
-    }
-    execlp(ui->shell, ui->shell, "-c", "echo $*", "1", NULL);
-    perror("Exec failed");
-    exit(1);
-  } else {        /* parent; read and check the child's output */
-    if (close(pipe_fd[1]) == -1) {
-      perror("(parent) Can't close pipe");
-      exit(1);
-    }
-    val[0] = '\0';
-    while ((p = read(pipe_fd[0], val, 3)) < 0) {
-      if (errno != EINTR) {
-	perror("Reading child's output failed");
-	exit(1);
-      }
-    }
-    if (p == 3)
-      return -1;
-    ui->arg0flag = (val[0] != '1');
-    return 0;
-  }
-}
-
-char *get_first(char *s)
-{
-  char *p;
-
-  p = strtok(s, ",");
-  if (p != NULL)
-    while (isspace(*p))
-      p++;
-  return p;
-}
-
-char *get_next()
-{
-  char *p;
-  
-  p = strtok((char *) NULL, ",");
-  if (p != NULL)
-    while (isspace(*p))
-      p++;
-  return p;
-}
-
-void init_arg0flag()
-{
-  if (resources.sh_list == NULL || !strcmp(resources.sh_list, "AUTO")) {
-    if (shell_test(&user) == -1) {
-      fprintf(stderr, "Xfm.BourneShells: AUTO test failed.\n");
-      exit(1);
-    }
+  if( sizeof(SortType) > toVal->size ) {
+	  toVal->addr = (caddr_t) &d;
   } else {
-    char *p;
-
-    for (p = get_first(resources.sh_list); p != NULL; p = get_next()) {
-      if ((user.arg0flag = !strcmp(p, user.shell)))
-        return;
-    }
+	  *(SortType*)toVal->addr = d;
   }
+  toVal->size = sizeof(SortType);
+  return True;
 }
 
-#ifdef ENHANCE_USERINFO
-
-/* 
- * test if the file named '*name' is executable
- * *name ist copied to ui->shell in this case.
- *
- * return values:
- *   0	  ok (copy *name to ui->shell)
- *   1    path not a regular file
- *   2	  regular file, but no permission to execute
- */
-
-static int shellpath_not_valid(char *name,UserInfo *ui)
+/*---------------------------------------------------------------------------*/
+ 
+static Boolean CvtStringToExpandedString(
+		UNUSED(Display *dpy), 
+		UNUSED(XrmValue *args), UNUSED(Cardinal *n_args),
+		XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
 {
-  struct stat	buf;
-  int		rval;
+  char *n;
 
-  if (stat(name,&buf) || ! S_ISREG(buf.st_mode)) rval=1;
-  else if (ui->uid==buf.st_uid) rval=(buf.st_mode & S_IXUSR ? 0:2); 
-  else if (ui->gid==buf.st_gid) rval=(buf.st_mode & S_IXGRP ? 0:2); 
-  else rval = (buf.st_mode & S_IXOTH ? 0:2);
-
-  if (rval==2) fprintf(stderr,"%s exists, no permission to execute\n",name);
-  if (rval==0) strcpy(ui->shell,name);
-  return rval;
-}
-
-/* 
- * retrieve the name of shell form the
- * application resources and put it into ui->shell.
- * if this fails, try to read the environment var SHELL
- * and finally use "/bin/sh"
- *
- * returns 0 if no executable shell was found.
- */
-static int get_sh_from_resources(UserInfo *ui)
-{
-  char 	*p;
-
-  if (0!=(p=resources.sh_cmd)) {
-    if (!shellpath_not_valid(p,ui)) return 1;
+  if( fromVal->size > 0 && fromVal->addr[fromVal->size-1] == '\0' &&
+		  toVal->size == sizeof(String)) {
+	  n = home_and_env_expand(fromVal->addr);
+	  if( n != NULL ) {
+		  *data = NULL;
+		  *(String*)toVal->addr = n;
+		  return True;
+	  }
   }
+  *data = NULL;
+  XtStringConversionWarning(fromVal->addr, XtRExpandedString);
+  return False;
+} 
 
-  if (!(p = getenv("SHELL"))) p="/bin/sh";
-
-  return ! shellpath_not_valid(p,ui);
+static void CvtFreeExpandedString(
+		UNUSED(XtAppContext app), UNUSED(XrmValue *to),
+		XtPointer data,
+		UNUSED(XrmValue *args), UNUSED(Cardinal *n_args)) {
+	XtFree(data);
 }
-#endif
 
 /*---------------------------------------------------------------------------
   Main function
 ---------------------------------------------------------------------------*/
 
-#ifdef ENHANCE_TXT_FIELD
-void noWarnings(String msg)
+static void noWarnings(UNUSED(String msg))
 {
 }
-#endif
-
-#ifdef ENHANCE_TRANSLATIONS
-static char app_ver0[100], *app_ver;
-#endif
 
 int main(int argc, char *argv[])
 {
+  int r;
   char *s;
-#ifdef ENHANCE_TRANSLATIONS
-  char *p, *xfmversion = XFMVERSION;
-#endif
+  const char *xfmversion = XFMVERSION;
 
   progname = argv[0];
 
@@ -490,12 +408,13 @@ int main(int argc, char *argv[])
   user.uid = getuid();
   user.gid = getgid();
 
-#ifndef ENHANCE_USERINFO /* read app-resources first */
   if ((s = getenv("HOME")))
     strcpy(user.home,s);
   else
     getwd(user.home);
+  getwd(user.cwd);
 
+#ifndef ENHANCE_USERINFO /* read app-resources first */
   if ((s = getenv("SHELL")))
     strcpy(user.shell,s);
   else
@@ -508,7 +427,7 @@ int main(int argc, char *argv[])
 
   /* initialise the application and create the application shell */
 
-  aw.shell = XtAppInitialize(&app_context, "Xfm", options, XtNumber(options),
+  aw.shell = XtAppInitialize(&app_context, XFM_APP_CLASS, options, XtNumber(options),
 			     &argc, argv, fallback_resources, shell_args,
 			     XtNumber(shell_args) );
 
@@ -521,10 +440,15 @@ int main(int argc, char *argv[])
     abortXfm("Couldn't mark display connection as close-on-exec");
 
   /* register resource converters */
-  XtAppAddConverter(app_context, XtRString, XtRDisplayType, 
-		    CvtStringToDisplayType, NULL, ZERO);
-  XtAppAddConverter(app_context, XtRString, XtRSortType, 
-		    CvtStringToSortType, NULL, ZERO);
+  XtSetTypeConverter(XtRString, XtRDisplayType, 
+		    CvtStringToDisplayType, NULL, ZERO,
+		    XtCacheAll, NULL);
+  XtSetTypeConverter(XtRString, XtRSortType, 
+		    CvtStringToSortType, NULL, ZERO,
+		    XtCacheAll, NULL);
+  XtSetTypeConverter(XtRString, XtRExpandedString, 
+		    CvtStringToExpandedString, NULL, ZERO,
+		    XtCacheAll|XtCacheRefCount,CvtFreeExpandedString);
 
   /* get the application resources */
   XtGetApplicationResources(aw.shell, &resources, resource_list,
@@ -536,26 +460,37 @@ int main(int argc, char *argv[])
     exit(0);
   }
 
-#ifdef ENHANCE_LOG
-  /* we need the 'tick' bitmap already here */
-  readBitmaps();
-  (void)FmCreateLog(aw.shell,resources.button_font);
-#endif
+  /* initialise the utilities module */
+  initUtils();
 
+  resources.symlinkdir = home_and_env_expand(resources.symlinkdir);
+  if( !mime_start_init(resources.symlinkdir) )
+	  abortXfm("error creating start directory");
 
 #ifdef ENHANCE_USERINFO
-  getwd(user.cwd);
-  if (!(s = getenv("HOME"))) s=user.cwd;
-  strcpy(user.home,s);
+  if (resources.shell == NULL || resources.shell[0] == NULL ||
+		  resources.shell[0][0] == '\0') {
+	  const char *shell = getenv("SHELL");
 
-  if (!get_sh_from_resources(&user)) {
-	fprintf(stderr, "%s%s\n",
-			"no executable shell found in ",
-			"(resources, environment, /bin/sh)");
-	exit(1);
+	  resources.shell = (const char **)XtMalloc(3*sizeof(char*));
+	  if (shell != NULL)
+	  	resources.shell[0] = (String)shell;
+	  else
+	  	resources.shell[0] = "/bin/sh";
+	  resources.shell[1] = "-c";
+	  resources.shell[2] = NULL;
   }
 #endif
 
+  r = mime_init();
+  if( r <= -2 )
+	  abortXfm("Out of Memory");
+  readBitmaps();
+
+#ifdef ENHANCE_LOG
+  (void)FmCreateLog(aw.shell,resources.button_font);
+#endif
+
   /* default is to launch both application and file manager */
   if (!resources.appmgr && !resources.filemgr)
     resources.appmgr = resources.filemgr = True;
@@ -563,80 +498,44 @@ int main(int argc, char *argv[])
   /* set the multi-click time */
   XtSetMultiClickTime(XtDisplay(aw.shell), resources.double_click_time);
 
-  /* initialise the utilities module */
-  initUtils();
-
   /* set up signal handlers */
-  sigcld.sa_handler = sigcldHandler;
-  sigemptyset(&sigcld.sa_mask);
-  sigcld.sa_flags = 0;
-  sigaction(SIGCHLD,&sigcld,NULL);
   sigterm.sa_handler = quit;
   sigemptyset(&sigterm.sa_mask);
   sigterm.sa_flags = 0;
   sigaction(SIGTERM,&sigterm,NULL);
+  if( !execute_init(app_context) ) {
+	  exit(EXIT_FAILURE);
+  }
 
-  /* check the user's shell; needs signal handlers (to avoid a zombie) */
-  init_arg0flag();
-
-#ifndef ENHANCE_LOG
-  /* create all the bitmaps & cursors needed */
-  readBitmaps();
-#endif
-  
   /* Set the icon for the application manager */
-  XtVaSetValues(aw.shell, XtNiconPixmap, bm[APPMGR_BM], NULL);
-  XtVaSetValues(aw.shell, XtNiconMask, bm[APPMGRMSK_BM], NULL);
+  XtVaSetValues(aw.shell, XtNiconPixmap, bm_appmgr, NULL);
+  XtVaSetValues(aw.shell, XtNiconMask, bm_appmgrmsk, NULL);
 
   /* create the main popup shells */
   createMainPopups();
 
 #ifdef ENHANCE_TRANSLATIONS
-  /* we don't want to define all the translations as fallback_resources
-   * rather we give up if the application defaults are not properly
-   * installed. (appDefsVersion should be set by the app. defs. file to
-   * the version nr. matching XFMVERSION)
-   */
-  /*strip eventual " */
-  if (resources.app_defs_version)
-    strcpy(app_ver0, resources.app_defs_version);
-  else
-    app_ver0[0] = '\0';
-  app_ver = app_ver0;
-  if (*app_ver==(char)'"') app_ver++;
-  if (p=strrchr(app_ver,'"')) *p=0;
-  if (strcmp(app_ver,xfmversion)) {
-    if (*app_ver ==(char)0)
-	error("Sorry: Appl. Defaults Not Found",
-	      "check XFILESEARCHPATH and friends");
-    else {
-	p=XtMalloc(128);
-	sprintf(p,"found: %s, needed: %s",
-	        app_ver,xfmversion);
-	error("Appl. Defaults: wrong version number",p);
-        XTFREE(p);
-    }
+  if (strcmp(resources.app_defs_version, XFMVERSION)) {
+    if (resources.app_defs_version[0] == '\0')
+	error("Sorry: Xfm.appDefsVersion was not defined!\n","perhaps Appl. Defaults are broken or too old!\n");
+    else
+	error("Appl. Defaults: wrong version number (need " XFMVERSION "): ",resources.app_defs_version);
     exit(1);
   }
 #endif
 
-  /* construct the config file names */
-  strcpy(resources.app_file, resources.app_file_r);
-  fnexpand(resources.app_file);
-  strcpy(resources.main_app_file, resources.app_file);
-  strcpy(resources.app_dir, resources.app_dir_r);
-  fnexpand(resources.app_dir);
-  strcpy(resources.app_clip, resources.app_clip_r);
-  fnexpand(resources.app_clip);
-  strcpy(resources.cfg_file, resources.cfg_file_r);
-  fnexpand(resources.cfg_file);
-  strcpy(resources.dev_file, resources.dev_file_r);
-  fnexpand(resources.dev_file);
-#ifdef MAGIC_HEADERS
-  strcpy(resources.magic_file, resources.magic_file_r);
-  fnexpand(resources.magic_file);
-#endif
-
+#ifdef SYSTEMWIDE_DEFAULTS
+  /* check if config-dir is missing but should contain clipfile */
+  if (!exists(resources.app_dir)) {
+    if (strncmp(resources.app_dir,resources.app_clip,
+                                  strlen(resources.app_dir))==0 
+        && ( resources.app_clip[strlen(resources.app_dir)] =='/' ||
+	    (strlen(resources.app_dir)>0 && 
+	     resources.app_clip[strlen(resources.app_dir)-1] == '/'))) {
+      mkdir(resources.app_dir,0777);
+    }
+  }
+#else
   /* check for config dir and run xfm.install if appropriate */
   if (!exists(resources.app_dir))
     if (confirm("It appears that you are running xfm for the first time.",
@@ -645,17 +544,28 @@ int main(int argc, char *argv[])
       system("xfm.install -n");
     else if (aborted)
       exit(1);
+#endif
 
+#ifdef SYSTEMWIDE_DEFAULTS
+  /* if the main_app_file is not found, use the system_main_app_file
+   * instead. (Only change app_file and not main_app_file, to
+   * allow adding a user-file while running). */
+  if (!exists(resources.main_app_file))
+    aw.app_file = XtNewString(resources.system_main_app_file);
+  else
+#endif
+    aw.app_file = XtNewString(resources.main_app_file);
+  
   /* initialise the applications module & create the window */
   if (resources.appmgr) {
-    readApplicationData(resources.app_file);
+    readApplicationData();
     createApplicationWindow();
     createApplicationDisplay();
   } else {
 	/* create a dummy window to store the WM_COMMAND property */
 	XtVaSetValues(aw.shell,
 		XtNmappedWhenManaged,False,
-		0);
+		(char*)0);
   }
   /* still realize and set the WM properties. the WM needs
    * a window around and kwm is even able to restart an app
@@ -686,22 +596,21 @@ int main(int argc, char *argv[])
   if (resources.update_interval > 0)
     XtAppAddTimeOut(app_context, resources.update_interval, timeoutCb, NULL);
 
-#ifdef ENHANCE_TXT_FIELD
   /* those 'text cursor out of range' messages from the TextField widget kept
      annoying me, so I switch Xt warnings off here --AG */
   XtAppSetWarningHandler(app_context, noWarnings);
-#endif
 
   /* collect & process events */
   XtAppMainLoop(app_context);
+  mime_start_done(resources.symlinkdir);
   return 0;
 }
 
 /*---------------------------------------------------------------------------*/
 
-void quit()
+void quit(UNUSED(int unused))
 {
-  int d;
+  unsigned int d;
 
   for (d = 0; d < n_devices; d++)
     if (devs[d].mounted) {
@@ -709,6 +618,7 @@ void quit()
       umountDev(d);
     }
 
+  mime_start_done(resources.symlinkdir);
   XtDestroyApplicationContext(app_context);
   exit(0);
 }
