@@ -1,5 +1,7 @@
-/*-----------------------------------------------------------------------------
-  Module FmUtils.c
+/*
+ *  This file is part of X-File Manager XFM
+ *  ----------------------------------------------------------------------
+  FmUtils.c
 
   (c) Simon Marlow 1990-1993
   (c) Albert Graef 1994
@@ -8,31 +10,47 @@
     (ender@ee.WPI.EDU), integrated Mar 24 1995, AG
 
   modified 7-1997 by strauman@sun6hft.ee.tu-berlin.de to add
-  different enhancements (see README-NEW).
+  different enhancements (see README-1.4).
 
+  modified 2004,2005,2006 by Bernhard R. Link (see Changelog)
 
   General utility functions for creating menus, buttons, questions,
   and functions for desensetising and 'ticking' menu entries.
------------------------------------------------------------------------------*/
+ *  ----------------------------------------------------------------------
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+#include <xfmconfig.h>
+
+#include <assert.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
-#include <X11/Xaw/MenuButton.h>
-#include <X11/Xaw/SimpleMenu.h>
-#include <X11/Xaw/SmeLine.h>
-#include <X11/Xaw/SmeBSB.h>
-#include <X11/Xaw/Command.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/Label.h>
-#include <X11/Xaw/Box.h>
-#ifdef ENHANCE_TXT_FIELD
+#include <X11/Xaw3d/MenuButton.h>
+#include <X11/Xaw3d/SimpleMenu.h>
+#include <X11/Xaw3d/SmeLine.h>
+#include <X11/Xaw3d/SmeBSB.h>
+#include <X11/Xaw3d/Command.h>
+#include <X11/Xaw3d/Form.h>
+#include <X11/Xaw3d/Label.h>
+#include <X11/Xaw3d/Box.h>
 #include "TextField.h"
 #include "FocusForm.h"
-#else
-#include <X11/Xaw/AsciiText.h>
-#endif
 
+#include "FmVersion.h"
+#include "global.h"
 #include "Am.h"
 #include "Fm.h"
 
@@ -94,12 +112,7 @@ static Arg text_args[] = {
   { XtNbottom, XtChainTop },
   { XtNleft, XtChainLeft },
   { XtNright, XtChainRight },
-#ifndef ENHANCE_TXT_FIELD
-  { XtNeditType, XawtextEdit },
-  { XtNtype, XawAsciiString },
-#else
   { XtNfocusInterest, True },
-#endif
   { XtNuseStringInPlace, (XtArgVal) True },
 };
 
@@ -155,13 +168,13 @@ void initUtils()
 /* specifed items; return the list of item widgets                           */
 /*****************************************************************************/
 
-Widget *createFloatingMenu(String menu_name,
-			   MenuItemRec *items, Cardinal n_items, 
+Widget *createFloatingMenu(const char *menu_name,
+			   const MenuItemRec *items, Cardinal n_items, 
 			   Dimension left_margin, Widget parent, 
 			   XtPointer client_data,
 			   Widget *menu_widget)
 {
-  int i;
+  Cardinal i;
   Widget *item_widgets;
   
   item_widgets = (Widget *) XtMalloc(n_items * sizeof(Widget));
@@ -203,11 +216,11 @@ Widget *createFloatingMenu(String menu_name,
 /* specifed items                                                            */
 /*****************************************************************************/
 
-Widget *createMenu(String menu_name, String menu_label, MenuItemList items,
-		   Cardinal n_items, Dimension left_margin,
-		   Widget parent, XtPointer client_data)
+Widget *createMenu(const char *menu_name, const char *menu_label,
+		const MenuItemRec *items, Cardinal n_items,
+		Dimension left_margin, Widget parent, XtPointer client_data)
 {
-  int i;
+  Cardinal i;
   Widget menu_widget, button_widget, *item_widgets;
   
   item_widgets = (Widget *) XtMalloc(n_items * sizeof(Widget));
@@ -249,10 +262,10 @@ Widget *createMenu(String menu_name, String menu_label, MenuItemList items,
 /*****************************************************************************/
 
 
-Widget *createButtons(ButtonList buttons, Cardinal n_buttons, Widget parent,
+Widget *createButtons(const ButtonRec *buttons, Cardinal n_buttons, Widget parent,
 		   XtPointer client_data)
 {
-  int i;
+  Cardinal i;
   Widget *button_widgets;
   
   button_widgets = (Widget *) XtMalloc(n_buttons * sizeof(Widget));
@@ -268,6 +281,63 @@ Widget *createButtons(ButtonList buttons, Cardinal n_buttons, Widget parent,
   return button_widgets;
 }
 
+void popdownPopup(struct PopupDialog *p)
+{
+	XtPopdown(p->shell);
+}
+void destroyPopup(struct PopupDialog *p)
+{
+	Cardinal i;
+	for (i=0; i < p->count ; i++) {
+		XTFREE(p->fields[i].answer);
+	}
+	XtDestroyWidget(p->shell);
+	XTFREE(p);
+}
+
+void reloadPopupData(struct PopupDialog *p)
+{
+	Cardinal i;
+	for (i=0; i<p->count; i++) {
+		XtVaSetValues(p->fields[i].widget, 
+				XtNstring, (XtArgVal)p->fields[i].answer,
+				XtNinsertPosition, (XtArgVal)0,
+				NULL);
+	}
+}
+bool setPopupData(struct PopupDialog *p, Cardinal i, const char *value)
+{
+	size_t l,m;
+	struct PopupDialogField *f;
+
+	assert(i < p->count);
+	assert(value != NULL);
+	l = strlen(value);
+	m = p->questions[i].length;
+	f = p->fields + i;
+
+	if (l>=m) {
+		memcpy(f->answer,value,m-1);
+		f->answer[m-1] = '\0';
+		XtVaSetValues(f->widget,
+				XtNstring, (XtArgVal)f->answer,
+				XtNinsertPosition, (XtArgVal)0,
+				NULL);
+		return true;
+	} else {
+		memcpy(p->fields[i].answer,value,l+1);
+		XtVaSetValues(f->widget, XtNstring,
+				(XtArgVal)f->answer, NULL);
+		return false;
+	}
+}
+
+void popupPopupByCursor(struct PopupDialog *p, void *data)
+{
+	p->data = data;
+	popupByCursor(p->shell, XtGrabExclusive);
+}
+
 /*****************************************************************************/
 /* Function: createPopupQuestions                                            */
 /* Arguments: name        :  The widget name for the shell                   */
@@ -285,33 +355,39 @@ Widget *createButtons(ButtonList buttons, Cardinal n_buttons, Widget parent,
 /* the bitmap, and a set of buttons underneath all this.                     */
 /*****************************************************************************/
 
-Widget createPopupQuestions(String name, String title, Pixmap bitmap,
-			    QuestionList questions, Cardinal n_questions,
-			    ButtonList buttons, Cardinal n_buttons,
+
+struct PopupDialog *createPopupQuestions(const char *name, const char *title,
+                            Pixmap bitmap,
+			    const QuestionRec *questions, Cardinal n_questions,
+			    const PDButtonRec *buttons, Cardinal n_buttons,
 			    Cardinal dflt_button)
 {
-  int i, l;
-  Widget form_widget, box_widget, bitmap_widget = NULL, shell,
+  Cardinal i;
+  int l;
+  struct PopupDialog *popup;
+  Widget form_widget, box_widget, bitmap_widget = NULL,
     vert = NULL, horiz = NULL;
-#ifdef ENHANCE_POP_ACCEL
   Widget *button_widgets;
+#ifdef ENHANCE_POP_ACCEL
 #ifndef ENHANCE_TRANSLATIONS
   int flag;
 #endif
 #endif
+  popup = (void*)XtMalloc(sizeof(struct PopupDialog)+n_questions*sizeof(struct PopupDialogField));
+  popup->questions = questions;
+  popup->count = n_questions;
+  for (i = 0; i < n_questions ; i++) {
+	  popup->fields[i].answer = XtCalloc(1,questions[i].length+1);
+  }
   /* create popup shell */
   shell_args[0].value = (XtArgVal) title;
-  shell = XtCreatePopupShell(name, transientShellWidgetClass, aw.shell,
+  popup->shell = XtCreatePopupShell(name, transientShellWidgetClass, aw.shell,
 			     shell_args, XtNumber(shell_args) );
 
   /* create form */
   form_args[0].value = PADDING;
-#ifdef ENHANCE_TXT_FIELD
-  form_widget = XtCreateManagedWidget("popup form", focusFormWidgetClass, shell, 
-#else
-  form_widget = XtCreateManagedWidget("popup form", formWidgetClass, shell, 
-#endif
-				      form_args, XtNumber(form_args) );
+  form_widget = XtCreateManagedWidget("popup form", focusFormWidgetClass,
+		  popup->shell, form_args, XtNumber(form_args) );
 
   /* create bitmap */
   if (bitmap != None) {
@@ -345,14 +421,10 @@ Widget createPopupQuestions(String name, String title, Pixmap bitmap,
       text_args[0].value = (XtArgVal) horiz;
       text_args[1].value = (XtArgVal) vert;
     }      
-    text_args[2].value = (XtArgVal) questions[i].value;
+    text_args[2].value = (XtArgVal) popup->fields[i].answer;
     text_args[3].value = (XtArgVal) questions[i].length;
-    vert = questions[i].widget = XtCreateManagedWidget("text", 
-#ifdef ENHANCE_TXT_FIELD
+    vert = popup->fields[i].widget = XtCreateManagedWidget("text", 
 	   textFieldWidgetClass, form_widget, text_args, XtNumber(text_args));
-#else
-	   asciiTextWidgetClass, form_widget, text_args, XtNumber(text_args));
-#endif
   }
 
   if (buttons != NULL) {
@@ -360,17 +432,15 @@ Widget createPopupQuestions(String name, String title, Pixmap bitmap,
     button_box_args[1].value = (XtArgVal) vert;
     box_widget = XtCreateManagedWidget("button box", boxWidgetClass, 
 	        form_widget, button_box_args, XtNumber(button_box_args));
-#ifdef ENHANCE_POP_ACCEL
     button_widgets =
-#endif
-    createButtons(buttons, n_buttons, box_widget, NULL);
+    createButtons((const ButtonRec*)buttons, n_buttons, box_widget, popup);
 
 #ifdef ENHANCE_POP_ACCEL
     for (i = 0; i<n_questions; i++) {
-    	XtInstallAllAccelerators(questions[i].widget,form_widget);
+    	XtInstallAllAccelerators(popup->fields[i].widget,form_widget);
     }
     XtInstallAllAccelerators(form_widget,form_widget);
-    XtInstallAllAccelerators(shell,form_widget);
+    XtInstallAllAccelerators(popup->shell,form_widget);
 #ifndef ENHANCE_TRANSLATIONS
     for (i=0;i<n_buttons;i++) {
 	/* This allows the user to trigger the buttons
@@ -392,12 +462,13 @@ Widget createPopupQuestions(String name, String title, Pixmap bitmap,
     }
 #endif
 #endif
+    XTFREE(button_widgets);
   }
 
-  XtRealizeWidget(shell);
-  setWMProps(shell);
+  XtRealizeWidget(popup->shell);
+  setWMProps(popup->shell);
 
-  return shell;
+  return popup;
 }
 
 /*****************************************************************************/
@@ -532,60 +603,60 @@ void wakeUp(void)
 
 #define MAXVARSTRINGLEN MAXPATHLEN
 
-static enum { DontKnow, Ok, Cancel } dialog_flag;
-static Widget dialog;
+enum dialog_flag { DontKnow, Ok, Cancel };
 
-static void dialogOkCb(Widget w, XtPointer client_data, XtPointer call_data)
+static void dialogOkCb(UNUSED(Widget w), struct PopupDialog *p,
+                       UNUSED(void *call_data))
 {
-  XtPopdown(dialog);
-  dialog_flag = Ok;
+	popdownPopup(p);
+	*(enum dialog_flag*)p->data = Ok;
 }
 
 /*---------------------------------------------------------------------------*/
 
-static void dialogCancelCb(Widget w, XtPointer client_data, 
-			   XtPointer call_data)
+static void dialogCancelCb(UNUSED(Widget w), struct PopupDialog *p,
+			   UNUSED(void *call_data))
 {
-  XtPopdown(dialog);
-  dialog_flag = Cancel;
+	popdownPopup(p);
+	*(enum dialog_flag*)p->data = Cancel;
 }
 
 /*---------------------------------------------------------------------------*/
 
-static ButtonRec dialog_buttons[] = {
-  { "ok", "Ok", (FmCallbackProc *) dialogOkCb },
-  { "cancel", "Cancel", (FmCallbackProc *) dialogCancelCb }
+static const PDButtonRec dialog_buttons[] = {
+  { "ok", "Ok", dialogOkCb },
+  { "cancel", "Cancel", dialogCancelCb }
 };
 
 char *varPopup(Pixmap icon_bm, char *action)
 {
-  static char *act = NULL;
+  char *act = NULL;
   char *act1 = (char *)alloca(strlen(action)+1), *s, *t;
   char *str = (char *)alloca(strlen(action)+1);
   char **acts = NULL, **vars = NULL;
   int n_acts = 0, n_vars = 0;
   char *def_val;
-  char **vals = NULL;
+  const char **default_vals = NULL;
+  struct PopupDialog *dialog;
+  enum dialog_flag dialog_flag;
 
-  if (act) XTFREE(act);
-  act = NULL;
   strcpy(act1, action);
 
   for (s = split(act1, '%'); s; s = split(NULL, '%')) {
     acts = (char **)XTREALLOC(acts, (n_acts+1)*sizeof(char *));
     acts[n_acts++] = XtNewString(strparse(str, s, "\\%"));
-    if (t = split(NULL, '%')) {
+    if ((t = split(NULL, '%'))) {
       vars = (char **)XTREALLOC(vars, (n_vars+1)*sizeof(char *));
       vars[n_vars] = XtNewString(strparse(str, t, "\\%"));
       /* Check string for default value character */
-      vals = (char **)XTREALLOC(vals, (n_vars+1)*sizeof(char *));
-      vals[n_vars] = (char *)XtMalloc(MAXVARSTRINGLEN);
+      default_vals = (const char **)XTREALLOC(default_vals, (n_vars+1)*sizeof(char *));
       if ((def_val = strstr(vars[n_vars], kDefaultValueMarker)) == NULL) {
-	strcpy(vals[n_vars++], kDefaultValue);
+	default_vals[n_vars] = kDefaultValue;
       } else {
 	def_val[0] = '\0'; /* Separate label and default value */
-	strcpy(vals[n_vars++], def_val + strlen(kDefaultValueMarker));
+	default_vals[n_vars] = def_val + strlen(kDefaultValueMarker);
       }
+      n_vars++;
     } else
       break;
   }
@@ -598,19 +669,19 @@ char *varPopup(Pixmap icon_bm, char *action)
     dialog_questions = (QuestionRec *)XtMalloc(n_vars*sizeof(QuestionRec));
     for (i = 0; i < n_vars; i++) {
       dialog_questions[i].label = vars[i];
-      dialog_questions[i].value = vals[i];
       dialog_questions[i].length = MAXVARSTRINGLEN;
-      dialog_questions[i].widget = NULL;
     }
 
     dialog = createPopupQuestions("dialog", "Parameter Dialog", icon_bm,
 				  dialog_questions, n_vars, dialog_buttons,
 				  XtNumber(dialog_buttons),
 				  XtNumber(dialog_buttons)-1);
-    popupByCursor(dialog, XtGrabExclusive);
-
+    for (i = 0; i < n_vars; i++) {
+	    setPopupData(dialog, i, default_vals[i]);
+    }
+    popupPopupByCursor(dialog, &dialog_flag);
+    
     dialog_flag = DontKnow;
-
     do {
       XtAppNextEvent(app_context, &e);
       XtDispatchEvent(&e);
@@ -618,11 +689,12 @@ char *varPopup(Pixmap icon_bm, char *action)
 
     if (dialog_flag == Ok)
       for (l = i = 0; i < n_acts; i++) {
-	int l1 = strlen(acts[i]), l2 = i<n_vars?strlen(vals[i]):0;
+	size_t l1 = strlen(acts[i]), l2 = i<n_vars?strlen(getPopupAnswer(dialog,i)):0;
 	act = (char *)XTREALLOC(act, l+l1+l2+1);
-	strcpy(act+l, acts[i]);
-	if (l2) strcpy(act+l+l1, vals[i]);
+	memcpy(act+l, acts[i], l1);
+	if (l2) memcpy(act+l+l1, getPopupAnswer(dialog,i), l2);
 	l += l1+l2;
+	act[l] = '\0';
       }
 
     XTFREE(dialog_questions);
@@ -630,14 +702,12 @@ char *varPopup(Pixmap icon_bm, char *action)
       XTFREE(acts[i]);
     for (i = 0; i < n_vars; i++)
       XTFREE(vars[i]);
-    for (i = 0; i < n_vars; i++)
-      XTFREE(vals[i]);
-    XTFREE(acts); XTFREE(vars); XTFREE(vals);
-    XtDestroyWidget(dialog);
+    XTFREE(acts); XTFREE(vars); XTFREE(default_vals);
+    destroyPopup(dialog);
     return act;
   } else {
     if (n_acts) XTFREE(acts);
-    return action;
+    return XtNewString(action);
   }
 }
 
@@ -647,11 +717,12 @@ char *varPopup(Pixmap icon_bm, char *action)
    text below a pixmap */
 
 static Widget createPopupText(String name, String title, Pixmap bitmap,
-			      String *lines, Cardinal n_lines,
-			      ButtonList buttons, Cardinal n_buttons,
+			      const char **lines, Cardinal n_lines,
+			      const ButtonRec *buttons, Cardinal n_buttons,
 			      Cardinal dflt_button)
 {
-  int i, l;
+  Cardinal i;
+  int l;
   Widget form_widget, box_widget, bitmap_widget = NULL, shell,
     vert = NULL;
 #ifdef ENHANCE_POP_ACCEL
@@ -667,11 +738,7 @@ static Widget createPopupText(String name, String title, Pixmap bitmap,
 
   /* create form */
   form_args[0].value = 5;
-#ifdef ENHANCE_TXT_FIELD
   form_widget = XtCreateManagedWidget("popup form", focusFormWidgetClass, shell, 
-#else
-  form_widget = XtCreateManagedWidget("popup form", formWidgetClass, shell, 
-#endif
 				      form_args, XtNumber(form_args) );
   /* create bitmap */
   if (bitmap != None) {
@@ -744,23 +811,21 @@ static Widget createPopupText(String name, String title, Pixmap bitmap,
 /*---------------------------------------------------------------------------*/
 
 static Widget about = None;
-static Pixmap logo_bm;
 
-static void aboutOkCb(Widget w, XtPointer client_data, 
-		      XtPointer call_data)
+static void aboutOkCb(UNUSED(Widget w), UNUSED(XtPointer client_data), 
+		      UNUSED(XtPointer call_data))
 {
   XtPopdown(about);
 }
 
 /*---------------------------------------------------------------------------*/
 
-static ButtonRec about_buttons[] = {
-  { "ok", "Ok", (FmCallbackProc *) aboutOkCb },
+static const ButtonRec about_buttons[] = {
+  { "ok", "Ok", aboutOkCb },
 };
 
-static char ver_str[250];
-static String about_text[] = {
-  ver_str,
+static const char *about_text[] = {
+  "This is xfm version " XFMVERSION "." XFMMINORVERSION ,
   "Written by Simon Marlow, Albert Graef, Till Straumann, et al",
   "Copying policy: GPL Version 2 or later",
 };
@@ -769,9 +834,9 @@ static String about_text[] = {
 
 void aboutPopup(void)
 {
+  Pixmap logo_bm;
   if (about == None) {
-    sprintf(ver_str, "This is xfm version %s.%s", XFMVERSION,XFMMINORVERSION);
-    logo_bm = readIcon("XfmLogo.xpm", 0, 0);
+    logo_bm = loadIcon("XfmLogo.xpm", NULL, NULL);
     about = createPopupText("about", "About xfm", logo_bm,
 			    about_text, XtNumber(about_text), about_buttons,
 			    XtNumber(about_buttons),
