@@ -1,54 +1,26 @@
 /*
- *  $Id: FocusForm.c,v 1.1.1.1.4.1 2000/02/23 16:21:52 till Exp $
- *
+ *  This file is part of X-File Manager XFM
+ *  ----------------------------------------------------------------------
  *  The FocusForm Widget
- *  --------------------
- 
+ *
  *  Copyright (C) 1997  by Till Straumann   <strauman@sun6hft.ee.tu-berlin.de>
-
+ *
+ *  modified 2005,2006 by Bernhard R. Link (see Changelog)
+ *  ----------------------------------------------------------------------
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Library Public License as published by
+ *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
 
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Library Public License for more details.
+ *  GNU General Public License for more details.
 
- *  You should have received a copy of the GNU General Library Public License
+ *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- *  $Log: FocusForm.c,v $
- *  Revision 1.1.1.1.4.1  2000/02/23 16:21:52  till
- *   - merged albert graef's changes from p3 to p4
- *
- *  Revision 1.6  1999/10/18 22:37:56  cvs
- *   - Imakefile supports building a (static) library now.
- *   - all (resource etc.) strings go to strDefs.c if SEPARATE_STRINGS is defined.
- *
- *  Revision 1.5  1999/10/18 03:51:46  cvs
- *   - fixed TextField bug: when restoring to a string_in_place buffer,
- *     XtVaSetValues(tfw,XtNstring..) must be called with the buffer pointer
- *     (and not the backup).
- *   - fixed/updated doc in TextField.h
- *   - FocusForm paranoia: SetValues checks for being FocusForm subclass.
- *
- *  Revision 1.4  1998/10/30 22:18:57  till
- *  -fixed some bugs in TextField
- *  -C++ compliance for TextField.h
- *  -replace selected part of string
- *
- *  Revision 1.3  1998/05/10 22:59:52  till
- *  dummy action prints a message
- *
- *  Revision 1.2  1998/04/19 11:00:24  till
- *  started cvs logging
- *
-
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
-
 /* This is a subclass of the athena 'Form' widget */
 /* Copyright (c) 1987, 1988, 1994  X Consortium
 
@@ -62,6 +34,8 @@ furnished to do so, subject to the following conditions:
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
 */
+#include <xfmconfig.h>
+
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
 #include <X11/Xos.h>
@@ -70,8 +44,9 @@ all copies or substantial portions of the Software.
 #include <stdio.h>
 #include <ctype.h>
 
-#include <X11/Xaw/XawInit.h>
+#include <X11/Xaw3d/XawInit.h>
 
+#include "global.h"
 #include "FocusFormP.h"
 
 /* Macros */
@@ -82,28 +57,25 @@ all copies or substantial portions of the Software.
  XtIsManaged(widget))
 
 /* Methods */
-static void	Initialize();
-static Boolean	ConstraintSetVals();
-static void	ConstraintInitialize();
-static void	ChangeManaged();
-static void	Realize();
-static void	Destroy();
+static void 	Initialize(Widget treq,Widget tnew,ArgList args,Cardinal *narg);
+static Boolean 	ConstraintSetVals(Widget current,Widget request,Widget new,ArgList args,Cardinal *narg);
+static void 	ConstraintInitialize(Widget treq,Widget tnew,ArgList args,Cardinal *narg);
+static void 	ChangeManaged(Widget w);
+static void 	Realize(Widget w,XtValueMask *vm,XSetWindowAttributes *wa);
+static void 	Destroy(Widget w);
 
 /* Actions */
-static void	FocusTraverse();
-static void	FocusSet();
-static void	Detail_NotifyAncestor();
+static void 	FocusTraverse(Widget w,XEvent *ev,String *args,Cardinal *nargs);
+static void 	FocusSet(Widget w,XEvent *ev,String *args,Cardinal *nargs);
+static void 	Detail_NotifyAncestor(Widget w,XEvent *ev,String *args,Cardinal *nargs);
 
 /* Event Handlers */
-static void	RevertFocus();
+static void 	RevertFocus(Widget wid,XtPointer cld,XEvent *ev,Boolean *ctd);
 
 /* private	Procedures */
-static int	FindCompositeChild();
-#ifdef notused
-static int	FindCompositeNamedChild();
-#endif
-static Widget	FindFocusFormAncestor();
-static Boolean  CanSetFocus();
+static int 	FindCompositeChild(Widget parent,Widget wid);
+static Widget 	FindFocusFormAncestor(Widget widget);
+static Boolean 	CanSetFocus(Widget w);
 
 #ifdef DEBUG
 static void dummy(Widget w, XEvent *ev, String *a, Cardinal *na)
@@ -221,10 +193,8 @@ FocusFormClassRec focusFormClassRec = {
 WidgetClass focusFormWidgetClass = (WidgetClass)&focusFormClassRec;
 
 
-static void Initialize(treq,tnew,args,narg)
-Widget treq,tnew;
-ArgList args;
-Cardinal *narg;
+static void Initialize(UNUSED(Widget treq), Widget tnew,
+                       UNUSED(ArgList args), UNUSED(Cardinal *narg))
 {
  FocusFormWidget ffw=(FocusFormWidget)tnew;
  ffw->focusForm.focus_child=(Widget)0;
@@ -233,17 +203,16 @@ Cardinal *narg;
    XtAddEventHandler(tnew,LeaveWindowMask,False,RevertFocus,0);
 }
 
-static void ConstraintInitialize(treq,tnew)
-Widget treq,tnew;
+static void ConstraintInitialize(UNUSED(Widget treq), Widget tnew,
+                                 UNUSED(ArgList args), UNUSED(Cardinal *narg))
 {
  tnew->core.sensitive=((FocusFormConstraints)(tnew->core.constraints))->focusForm.sensitive;
 }
 
 
-static Boolean ConstraintSetVals(current,request,new,args,narg)
-Widget current,request,new;
-ArgList args;
-Cardinal *narg;
+static Boolean ConstraintSetVals(Widget current, UNUSED(Widget request),
+                                 Widget new,
+				 UNUSED(ArgList args), UNUSED(Cardinal *narg))
 {
 Boolean				loose_focus;
 Cardinal			dummy=0;
@@ -263,16 +232,12 @@ if (XtIsSubclass(XtParent(current),focusFormWidgetClass)) {
 return True;
 }
 
-static void Destroy(w)
- Widget w;
+static void Destroy(Widget w)
 {
  XtRemoveEventHandler(w,LeaveWindowMask,False,RevertFocus,0);
 }
 
-static void Realize(w,vm,wa)
-Widget w;
-XtValueMask vm;
-XSetWindowAttributes *wa;
+static void Realize(Widget w,XtValueMask *vm,XSetWindowAttributes *wa)
 {
 register Widget wtmp;
 /* Do what must be done */
@@ -287,11 +252,11 @@ if (!FindFocusFormAncestor(w)) {
 }
 
 
-static void ChangeManaged(w)
- Widget w;
+static void ChangeManaged(Widget w)
 {
 FocusFormWidget ffw=(FocusFormWidget)w;
-int		focus_num;
+int     	focus_num;
+Cardinal	i;
 Cardinal	narg;
 register Widget top,wtmp;
 Widget		winit;
@@ -309,9 +274,9 @@ if (! ffw->focusForm.initialized) {
   if (! (name && (winit=XtNameToWidget(top,name)) && 
          w==XtParent(winit) && TAKES_FOCUS(winit)) ) {
    /* no initial widget?, take first */
-   for (focus_num=0;focus_num<ffw->composite.num_children;focus_num++) {
- 	if (TAKES_FOCUS(ffw->composite.children[focus_num])) {
-	  winit=ffw->composite.children[focus_num];
+   for (i=0;i<ffw->composite.num_children;i++) {
+ 	if (TAKES_FOCUS(ffw->composite.children[i])) {
+	  winit=ffw->composite.children[i];
 	  break;
 	}
    }
@@ -348,11 +313,7 @@ if (focus_num>=0 && !XtIsManaged(ffw->focusForm.focus_child)) {
  * of a FocusIn/Out event is not NotifyAncestor.
  */
 
-static void Detail_NotifyAncestor(w,ev,args,nargs)
-Widget		w;
-XEvent		*ev;
-String		*args;
-Cardinal	*nargs;
+static void Detail_NotifyAncestor(Widget w,XEvent *ev,String *args,Cardinal *nargs)
 {
 if (*nargs<1) {
  XtAppWarning(XtWidgetToApplicationContext(w),
@@ -373,11 +334,7 @@ if ( (ev->type==FocusIn || ev->type==FocusOut) ){
 XtCallActionProc(w,args[0],ev,args+1,*nargs+1);
 }
 
-static void FocusSet(w,ev,args,nargs)
-Widget		w;
-XEvent		*ev;
-String		*args;
-Cardinal	*nargs;
+static void FocusSet(Widget w,XEvent *ev,String *args,Cardinal *nargs)
 {
 Widget top,tmpw,new_focus_widget,tmpw1;
 FocusFormWidget ffw;
@@ -420,11 +377,8 @@ if (new_focus_widget && TAKES_FOCUS(new_focus_widget)) {
 
 }
 
-static void FocusTraverse(w,ev,args,nargs)
-Widget		w;
-XEvent		*ev;
-String		*args;
-Cardinal	*nargs;
+static void FocusTraverse(Widget w, UNUSED(XEvent *ev),
+                          String *args, Cardinal *nargs)
 {
 Widget			new_focus_widget,old_focus_widget;
 register Widget		top,wtmp;
@@ -461,7 +415,7 @@ if (num_children==0) {
 }
 
 old_focus_widget=comp->focusForm.focus_child;
-new_focus=FindCompositeChild(comp,old_focus_widget);
+new_focus=FindCompositeChild((Widget)comp,old_focus_widget);
 if (new_focus<0) {
 	/* lost track */
 	comp->focusForm.focus_child=0;
@@ -494,8 +448,7 @@ XtWarning("FocusForm (FocusTraverse()): Focus could not be changed");
 /* finds widget in composite's children-list.
  * returns -1 if nothing found
  */
-static int FindCompositeChild(parent,wid)
-Widget parent,wid;
+static int FindCompositeChild(Widget parent,Widget wid)
 {
 CompositeWidget comp=(CompositeWidget)parent;
 int	n;
@@ -525,8 +478,7 @@ return n;
 }
 #endif
 
-static Widget FindFocusFormAncestor(widget)
- Widget widget;
+static Widget FindFocusFormAncestor(Widget widget)
 {
  do {
   if ( XtIsShell(widget=XtParent(widget)) ) return 0;
@@ -534,8 +486,7 @@ static Widget FindFocusFormAncestor(widget)
  return widget;
 }
 
-static Boolean CanSetFocus(w)
-Widget w;
+static Boolean CanSetFocus(Widget w)
 {
 char *mess;
 if (! (XtIsSubclass(w,focusFormWidgetClass) || 
@@ -553,11 +504,8 @@ if (! (XtIsSubclass(w,focusFormWidgetClass) ||
  * scope of what ffw can do. Therefore we set the focus
  * to the parent.
  */
-static void RevertFocus(wid,cld,ev,ctd)
- Widget wid;
- XtPointer *cld;
- XEvent *ev;
- Boolean *ctd;
+static void RevertFocus(Widget wid, UNUSED(XtPointer cld),
+                        XEvent *ev, UNUSED(Boolean *ctd))
 {
 if (ev->xcrossing.detail!=NotifyInferior)
   XtSetKeyboardFocus(XtParent(wid),XtParent(wid));
