@@ -395,7 +395,7 @@ static inline int mime_read_data(int *linenr,int *colnr,
 	size_t i = 0;
 	bool escaped = false;
 	size_t buflen = 1000;
-	char *buffer = malloc(128);
+	char *buffer = malloc(buflen);
 
 	if( buffer == NULL )
 		return -2;
